package com.baijiayun.groupclassui.chat;

import android.content.Context;
import android.graphics.drawable.Drawable;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import androidx.annotation.NonNull;
import androidx.core.content.ContextCompat;

import com.baijiayun.bjyutils.drawable.DrawableBuilder;
import com.baijiayun.bjyutils.kt.UtilsKt;
import com.baijiayun.groupclassui.R;
import com.baijiayun.groupclassui.global.EventKey;
import com.baijiayun.groupclassui.global.IRouter;
import com.baijiayun.livebase.utils.ThemeDataUtil;
import com.baijiayun.livecore.models.imodels.IMessageModel;
import com.baijiayun.liveuibase.widgets.chat.FloatChatMessageCommonAdapter;

/**
 * Created by yongjiaming on 4/2/21
 * 聊天飘窗消息adapter
 */
public class FloatChatMessageAdapter extends FloatChatMessageCommonAdapter {
    private final IRouter router;

    public FloatChatMessageAdapter(Context context, IRouter router) {
        super(context, router.getLiveRoom());
        this.router = router;
    }

    @Override
    public void setItemViewClickListener(View itemView) {
        itemView.setOnClickListener(v -> router.getSubjectByKey(EventKey.DisplayChat).onNext(true));
    }

    @Override
    public Drawable setItemBackground() {
        int dp6 = UtilsKt.getDp(6);
        return new DrawableBuilder()
                .solidColor(
                        ThemeDataUtil.getColorFromThemeConfigByAttrId(
                                getContext(),
                                R.attr.base_theme_window_bg_color
                        )
                )
                .strokeColor(ContextCompat.getColor(getContext(), R.color.bjysc_bg_stroke_10))
                .strokeWidth(UtilsKt.getDp(1))
                .cornerRadius(dp6)
                .build();
    }

    @NonNull
    @Override
    public FloatChatMessageCommonAdapter.TextViewHolder createTextViewHolder(ViewGroup parent) {
        return new TextViewHolder(LayoutInflater.from(parent.getContext()).inflate(R.layout.bjy_group_item_float_chat_text, parent, false));
    }

    @NonNull
    @Override
    public FloatChatMessageCommonAdapter.ImageViewHolder createImageViewHolder(ViewGroup parent) {
        return new ImageViewHolder(LayoutInflater.from(parent.getContext()).inflate(R.layout.bjy_group_item_float_chat_image, parent, false));
    }

    @Override
    public int maxChatCount() {
        return 3;
    }

    @Override
    public int setDisplayTime() {
        return 5000;
    }

    @Override
    public boolean shouldNameAppendToTextContent() {
        return true;
    }

    @Override
    public boolean filterWhenMsgAdd(@NonNull IMessageModel message) {
        return true;
    }

    static class TextViewHolder extends FloatChatMessageCommonAdapter.TextViewHolder {

        public TextViewHolder(@NonNull View itemView) {
            super(itemView);
            setContentTextView(itemView.findViewById(R.id.float_chat_content_tv));
        }
    }

    static class ImageViewHolder extends FloatChatMessageCommonAdapter.ImageViewHolder {

        public ImageViewHolder(@NonNull View itemView) {
            super(itemView);
            setContentImageView(itemView.findViewById(R.id.float_img_msg_iv));
            setUserNameTextView(itemView.findViewById(R.id.float_chat_user_tv));
        }
    }

}
