package com.baijiayun.groupclassui.container;

import android.content.Context;
import android.graphics.Color;
import android.os.Build;
import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.annotation.RequiresApi;
import androidx.constraintlayout.widget.ConstraintLayout;
import androidx.constraintlayout.widget.ConstraintSet;
import android.util.AttributeSet;
import android.view.ViewGroup;

import com.baijiayun.groupclassui.R;
import com.baijiayun.groupclassui.layer.BaseLayerOfProfessionalClass;
import com.baijiayun.groupclassui.layer.SeatLayer;
import com.baijiayun.groupclassui.layer.VideoDragLayer;
import com.baijiayun.livebase.context.LPConstants;


/**
 * Created by Shubo on 2018/9/13.
 */
public class SeatContainer extends BaseLayerOfProfessionalClass implements VideoContainerListener {
    private final String TAG = SeatContainer.class.getCanonicalName();
    private SeatLayer seatLayer;
    private VideoDragLayer videoDragLayer;

    public SeatContainer(@NonNull Context context) {
        super(context);
    }

    public SeatContainer(@NonNull Context context, @Nullable AttributeSet attrs) {
        this(context, attrs, 0);
    }

    public SeatContainer(@NonNull Context context, @Nullable AttributeSet attrs, int defStyleAttr) {
        super(context, attrs, defStyleAttr);
    }

    @RequiresApi(api = Build.VERSION_CODES.LOLLIPOP)
    public SeatContainer(@NonNull Context context, @Nullable AttributeSet attrs, int defStyleAttr, int defStyleRes) {
        super(context, attrs, defStyleAttr, defStyleRes);
    }


    @Override
    public boolean onRoomStatusChange(boolean isRoomActive) {
        boolean isLayerInit = super.onRoomStatusChange(isRoomActive);
        if(isLayerInit && !isRoomActive){
            release();
        }
        return true;
    }

    @Override
    public void initVideo() {
        seatLayer = new SeatLayer(getContext());
        videoDragLayer = new VideoDragLayer(getContext());
        seatLayer.setVideoDragLayer(videoDragLayer);
        if (router.getLiveRoom().getSmallClassTemplateType() == LPConstants.SmallClassTemplateType.ONEONONE) {
            post(() -> {
                if (videoDragLayer != null) {
                    videoDragLayer.setSeatContainerWidth(getMeasuredWidth());
                }
            });
        }else{
            post(() -> {
                if (videoDragLayer != null) {
                    videoDragLayer.setSeatContainerHeight(getMeasuredHeight());
                }
            });
        }
        addView(seatLayer, new ViewGroup.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.MATCH_PARENT));
    }


    public VideoDragLayer getVideoDragLayer() {
        return videoDragLayer;
    }

    public void switchLayoutMode(boolean isBoardLayout) {
        seatLayer.switchLayoutMode(isBoardLayout);
        ConstraintLayout.LayoutParams layoutParams = (ConstraintLayout.LayoutParams) getLayoutParams();
        //板书布局
        if (isBoardLayout) {
            if (router.getLiveRoom().getSmallClassTemplateType() == LPConstants.SmallClassTemplateType.DEFAULT) {
                //默认模板
                layoutParams.bottomToTop = R.id.activity_group_class_sync_container;
                layoutParams.bottomToBottom = -1;
            } else{
                //1v1模板
                layoutParams.topToBottom = R.id.activity_group_class_status_bar_container;
                layoutParams.bottomToBottom = ConstraintSet.PARENT_ID;
                layoutParams.endToEnd = ConstraintSet.PARENT_ID;
                setBackgroundColor(Color.TRANSPARENT);
            }
        } else {
            if (router.getLiveRoom().getSmallClassTemplateType() == LPConstants.SmallClassTemplateType.DEFAULT) {
                //默认模板
                layoutParams.bottomToTop = -1;
                layoutParams.bottomToBottom = ConstraintSet.PARENT_ID;
            } else {
                layoutParams.topToBottom = -1;
                layoutParams.topToTop = ConstraintSet.PARENT_ID;
            }
        }
        setLayoutParams(layoutParams);
    }

    @Override
    public void switchStudyRoomMode(LPConstants.StudyRoomMode roomMode) {
        // do nothing
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
        release();
    }

    @Override
    public void release() {
        if (videoDragLayer != null) {
            videoDragLayer.onDestroy();
        }
        if(seatLayer != null){
            removeAndDestroy(seatLayer);
        }
    }
}
