package com.baijiayun.groupclassui.container;

import com.baijiayun.groupclassui.window.toolbar.DrawWidthSettingWindow;
import com.baijiayun.livebase.context.LPConstants;
import com.baijiayun.liveuibase.base.BaseUIConstant.SelectSrc;
import com.baijiayun.liveuibase.widgets.toolbar.ColorSelectData;
import com.baijiayun.liveuibase.widgets.toolbar.WidthSelectData;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

/**
 * Created by yongjiaming on 2019/3/23
 */
public class ShapePaint {

    private HashMap<SelectSrc, Integer> mShapeColor;

    private HashMap<SelectSrc, Float> mDoodleStrokeWidth;

    private float mCustomStrokeWidth;

    private int mTextSize = 20;

    private LPConstants.ShapeType shapeType;

    private SelectSrc shapeSelectSrc;

    private LPConstants.PPTEditMode pptEditMode = LPConstants.PPTEditMode.Normal;
    private final List<IShapePaintListener> shapePaintListeners = new ArrayList<>();

    public ShapePaint(int mShapeColor, float mCustomStrokeWidth) {
        setCustomStrokeWidth(mCustomStrokeWidth);
        setDoodleStrokeWidth(new WidthSelectData(SelectSrc.Brush, DrawWidthSettingWindow.pencilPaintSize[1]));
        setDoodleStrokeWidth(new WidthSelectData(SelectSrc.Marker, DrawWidthSettingWindow.markerPaintSize[1]));
        setShapeColor(new ColorSelectData(SelectSrc.Graph, mShapeColor));
        setShapeColor(new ColorSelectData(SelectSrc.Text, mShapeColor));
        setShapeColor(new ColorSelectData(SelectSrc.Brush, mShapeColor));
        setShapeColor(new ColorSelectData(SelectSrc.Marker, mShapeColor));
    }

    public void setCustomStrokeWidth(float mCustomStrokeWidth) {
        this.mCustomStrokeWidth = mCustomStrokeWidth;
        for (IShapePaintListener shapePaintListener : shapePaintListeners) {
            shapePaintListener.onShapeCustomStrokeWidthChange(mCustomStrokeWidth);
        }
    }

    public void setShapeColor(ColorSelectData colorSelectData) {
        if (mShapeColor == null) {
            mShapeColor = new HashMap<>();
        }
        mShapeColor.put(colorSelectData.getSelectSrc(), colorSelectData.getSelectColor());
        for (IShapePaintListener shapePaintListener : shapePaintListeners) {
            shapePaintListener.onShapeColorChange(colorSelectData.getSelectColor());
        }
    }

    public void setDoodleStrokeWidth(WidthSelectData widthSelectData) {
        if (this.mDoodleStrokeWidth == null) {
            this.mDoodleStrokeWidth = new HashMap<>();
        }
        this.mDoodleStrokeWidth.put(widthSelectData.getSelectSrc(), widthSelectData.getWidth());
        for (IShapePaintListener shapePaintListener : shapePaintListeners) {
            shapePaintListener.onShapeDoodleStrokeWidthChange(widthSelectData.getWidth());
        }
    }

    public LPConstants.ShapeType getShapeType() {
        return shapeType;
    }

    public void setShapeType(SelectSrc selectSrc, LPConstants.ShapeType shapeType) {
        this.shapeType = shapeType;
        this.shapeSelectSrc = selectSrc;
        for (IShapePaintListener shapePaintListener : shapePaintListeners) {
            shapePaintListener.onShapeTypeChange(shapeType);
            Integer shapeColor = mShapeColor.get(selectSrc);
            if (shapeColor != null) {
                shapePaintListener.onShapeColorChange(shapeColor);
            }
            Float width = mDoodleStrokeWidth.get(selectSrc);
            if (width != null) {
                shapePaintListener.onShapeDoodleStrokeWidthChange(width);
            }
        }
    }

    public void setPPtEditMode(LPConstants.PPTEditMode pptEditMode) {
        this.pptEditMode = pptEditMode;
        for (IShapePaintListener shapePaintListener : shapePaintListeners) {
            shapePaintListener.onShapeEditModeChange(pptEditMode);
        }
    }

    public LPConstants.PPTEditMode getPPtEditMode() {
        return pptEditMode;
    }

    public void setTextSize(int textSize) {
        this.mTextSize = textSize;
        for (IShapePaintListener shapePaintListener : shapePaintListeners) {
            shapePaintListener.onShapeTextSizeChange(textSize);
        }
    }

    public void sendTextConfirmed(String beforeText, String textContent) {
        for (IShapePaintListener shapePaintListener : shapePaintListeners) {
            shapePaintListener.onSendDrawTextConfirmed(beforeText, textContent);
        }
    }

    public int getTextSize() {
        return mTextSize;
    }

    public float getCustomStrokeWidth() {
        return mCustomStrokeWidth;
    }

    public float getDoodleStrokeWidth() {
        Float width = mDoodleStrokeWidth.get(shapeSelectSrc);
        return  width == null ? 0 : width;
    }

    public int getShapeColor() {
        Integer color = mShapeColor.get(shapeSelectSrc);
        return color == null ? 0 : color;
    }

    public SelectSrc getShapeSelectSrc() {
        return shapeSelectSrc;
    }

    public void registerShapePaintListener(IShapePaintListener shapePaintListener) {
        shapePaintListeners.add(shapePaintListener);
    }

    public void unregisterShapePaintListener(IShapePaintListener shapePaintListener) {
        shapePaintListeners.remove(shapePaintListener);
    }

    public void removeAllListener() {
        shapePaintListeners.clear();
    }

    public interface IShapePaintListener {

        void onShapeCustomStrokeWidthChange(float strokeWidth);

        void onShapeDoodleStrokeWidthChange(float strokeWidth);

        void onShapeColorChange(int shapeColor);

        void onShapeTypeChange(LPConstants.ShapeType shapeType);

        void onShapeEditModeChange(LPConstants.PPTEditMode pptEditMode);

        void onShapeTextSizeChange(int textSize);

        void onSendDrawTextConfirmed(String beforeText, String content);
    }


}
