package com.baijiayun.groupclassui.dialog

import android.os.Bundle
import android.view.Gravity
import android.view.View
import android.view.WindowManager
import androidx.recyclerview.widget.LinearLayoutManager
import com.baijiayun.bjyutils.reactivex.RxUtils
import com.baijiayun.groupclassui.R
import com.baijiayun.groupclassui.base.BaseGroupDialogFragment
import com.baijiayun.groupclassui.global.EventCode
import com.baijiayun.groupclassui.global.IRouter
import com.baijiayun.groupclassui.user.ReplaceListRecyclerViewAdapter
import com.baijiayun.groupclassui.user.ReplacePresenter
import com.baijiayun.groupclassui.user.ReplaceUserWithStatus
import com.baijiayun.livebase.context.LPConstants
import com.baijiayun.livebase.models.imodels.IUserModel
import io.reactivex.Observable
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.disposables.Disposable
import kotlinx.android.synthetic.main.bjy_group_dialog_activeuser_full_and_replace.*
import java.util.concurrent.TimeUnit


class ActiveUserFullAndReplaceDialog(router: IRouter, private val userAdd: ReplaceUserWithStatus)
    : BaseGroupDialogFragment(), FullAndReplaceListener {
    private val presenter = ReplacePresenter(this, router)
    private var fullCount = presenter.router.liveRoom.maxActiveUsers
    private var curActiveCount: Int? = null
    private var lastOnlineUserCount: Int? = null
    private val replaceListRecyclerViewAdapter by lazy {
        context?.let { ReplaceListRecyclerViewAdapter(filterTeacherFromActiveUserList(), it, presenter) }
    }
    private var disposeTimer: Disposable? = null

    override fun getLayoutId(): Int = R.layout.bjy_group_dialog_activeuser_full_and_replace

    override fun init(savedInstanceState: Bundle?, arguments: Bundle?) {
        // do nothing
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        presenter.subscribe()
        replace_recycler.layoutManager = LinearLayoutManager(context)
        replace_recycler.adapter = replaceListRecyclerViewAdapter
        // 过滤老师
        var activeUserListCur = filterTeacherFromActiveUserList()
        lastOnlineUserCount = presenter.router.liveRoom.onlineUserVM.onlineUserList.size
        curActiveCount = activeUserListCur.size
        if (fullCount != curActiveCount) {
            remindVacancy()
        }
        // 取消
        replace_negative_tv.setOnClickListener {
            RxUtils.dispose(disposeTimer)
            dismissAllowingStateLoss()
        }
        // 替换
        replace_positive_tv.setOnClickListener {
            // 点击替换时需要更新activeList，保证时效性
            activeUserListCur = filterTeacherFromActiveUserList()
            activeUserListCur.let {
                if (replaceListRecyclerViewAdapter != null && replaceListRecyclerViewAdapter!!.getSelectedUserPosition() != -1) {
                    val selectedPosition = replaceListRecyclerViewAdapter!!.getSelectedUserPosition()
                    if (it.isNotEmpty() && selectedPosition <= it.size - 1) {
                        // 选择的是非空位
                        val selectedIUserModel = it[selectedPosition]
                        // user下台默认关闭音视频，不再发言
                        presenter.router.liveRoom.onlineUserVM.requestRemoveActiveUser(selectedIUserModel)
                        presenter.router.liveRoom.speakQueueVM.closeOtherSpeak(selectedIUserModel.userId)
                    }
                    if (userAdd.isHandsUp) {
                        if (userAdd.isResponder) {
                            // 抢答发言
                            presenter.router.liveRoom.speakQueueVM.controlRemoteUser(userAdd.iUserModel.userId, true, true)
                        } else {
                            // 举手发言只需要发送同意信令，对齐pc端
                            presenter.router.liveRoom.speakQueueVM.agreeSpeakApply(userAdd.iUserModel.userId)
                        }
                        userAdd.isHandsUp = false
                        userAdd.isResponder = false
                        presenter.router.getPublishSubjectByKey<ReplaceUserWithStatus>(EventCode.UserWhenFullActiveAdd).onNext(userAdd)
                    } else {
                        presenter.router.liveRoom.onlineUserVM.requestAddActiveUser(userAdd.iUserModel)
                    }
                    dismissAllowingStateLoss()
                }
            }
        }
        replace_exit_tv.setOnClickListener {
            RxUtils.dispose(disposeTimer)
            dismissAllowingStateLoss()
        }
    }

    // 空位提示
    private fun remindVacancy() {
        replace_tips_tv.text = context?.resources?.getString(R.string.bjy_group_activeuser_dialog_vacancy_tips)
        replaceListRecyclerViewAdapter?.notifyCurrentActiveUserChange(filterTeacherFromActiveUserList())
    }

    // 满员提示，在台上有空位并老师正在操作时，台下学生请求上台正好占据了空位
    private fun remindFull() {
        replace_tips_tv.text = context?.resources?.getString(R.string.bjy_group_activeuser_dialog_tips)
        replaceListRecyclerViewAdapter?.notifyCurrentActiveUserChange(filterTeacherFromActiveUserList())
    }

    private fun remindSelfActive() {
        if (disposeTimer != null) {
            return
        }
        replace_tips_container.visibility = View.GONE
        replace_positive_tv.visibility = View.GONE
        replace_negative_tv.visibility = View.GONE
        replace_countdown_container.visibility = View.VISIBLE
        replace_exit_sure_tv.visibility = View.VISIBLE
        replace_exit_sure_tv.setOnClickListener {
            RxUtils.dispose(disposeTimer)
            if (disposeTimer != null) {
                if (disposeTimer!!.isDisposed) {
                    dismissAllowingStateLoss()
                }
            } else {
                dismissAllowingStateLoss()
            }
        }
        context?.let {
            replace_countdown_mes_tv.text = it.resources.getString(R.string.bjy_group_activeuser_dialog_countdown_already_stage_mes, userAdd.iUserModel.name)
        }
        countTimer()
    }

    private fun remindSelfLeave() {
        // 避免回调两次remind方法覆盖
        if (disposeTimer != null) {
            return
        }
        replace_tips_container.visibility = View.GONE
        replace_positive_tv.visibility = View.GONE
        replace_negative_tv.visibility = View.GONE
        replace_countdown_container.visibility = View.VISIBLE
        replace_exit_sure_tv.visibility = View.VISIBLE
        replace_exit_sure_tv.setOnClickListener {
            RxUtils.dispose(disposeTimer)
            if (disposeTimer != null) {
                if (disposeTimer!!.isDisposed) {
                    dismissAllowingStateLoss()
                }
            } else {
                dismissAllowingStateLoss()
            }
        }
        context?.let {
            replace_countdown_mes_tv.text = it.resources.getString(R.string.bjy_group_activeuser_dialog_countdown_already_leave_mes, userAdd.iUserModel.name)
        }
        countTimer()
    }

    private fun remindSelfCancelHandsUp() {
        if (disposeTimer != null) {
            return
        }
        replace_tips_container.visibility = View.GONE
        replace_positive_tv.visibility = View.GONE
        replace_negative_tv.visibility = View.GONE
        replace_countdown_container.visibility = View.VISIBLE
        replace_exit_sure_tv.visibility = View.VISIBLE
        replace_exit_sure_tv.setOnClickListener {
            RxUtils.dispose(disposeTimer)
            if (disposeTimer != null) {
                if (disposeTimer!!.isDisposed) {
                    dismissAllowingStateLoss()
                }
            } else {
                dismissAllowingStateLoss()
            }
        }
        context?.let {
            val text = it.resources.getString(R.string.bjy_group_activeuser_dialog_countdown_cancel_handsup_mes, userAdd.iUserModel.name)
            replace_countdown_mes_tv.text = text
        }
        countTimer()
    }

    private fun countTimer() {
        RxUtils.dispose(disposeTimer)
        var count = 5
        disposeTimer = Observable.interval(0, 1, TimeUnit.SECONDS)
                .take(6)
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe {
                    context?.let {
                        if (count >= 1) {
                            replace_countdown_exit_tv.text = it.resources.getString(R.string.bjy_group_activeuser_dialog_countdown_exit_text, count)
                            count -= 1
                        } else {
                            dismissAllowingStateLoss()
                        }
                    }
                }
    }

    /**
     * dialog加载后供外部调用
     * 上下台、学生离开
     * 取消举手、举手超时
     *
     * tips: 需要保证curActiveNumber的正确性
     */
    private fun remindActiveUserChange() {
        val allOnlineUser = presenter.router.liveRoom.onlineUserVM.onlineUserList
        val activeList = filterTeacherFromActiveUserList()
        // 离开触发
        if (lastOnlineUserCount!! > allOnlineUser.size) {
            lastOnlineUserCount = allOnlineUser.size
            if (allOnlineUser.contains(userAdd.iUserModel)) {
                // 离开的学生不是申请人
                if (curActiveCount!! > activeList.size) {
                    // 台上人离开
                    remindVacancy()
                    curActiveCount = activeList.size
                    return
                }
            } else {
                // 申请人离开提示
                remindSelfLeave()
            }
            return
        }

        if (userAdd.isHandsUp) {
            if (curActiveCount == activeList.size) {
                // 取消举手、举手超时提示
                if (isApplicantAlreadyActive(activeList)) {
                    // 老师操作时助教操作上台，getObservableOfSpeakResponse回调
                    remindSelfActive()
                } else {
                    remindSelfCancelHandsUp()
                }
                return
            }
        }

        // 上下台触发
        if (isApplicantAlreadyActive(activeList)) {
            // 申请人已上台提示
            remindSelfActive()
        } else {
            // 非申请人上下台
            if (fullCount > activeList.size) {
                remindVacancy()
            } else {
                remindFull()
            }
        }
        curActiveCount = activeList.size
    }

    private fun filterTeacherFromActiveUserList(): MutableList<IUserModel> {
        // 过滤老师并且将助教前移
        val activeUserListCur = presenter.router.liveRoom.onlineUserVM.activeUserList
        val ass: MutableList<IUserModel> = ArrayList()
        val stu: MutableList<IUserModel> = ArrayList()
        activeUserListCur?.let {
            for (iUserModel in it) {
                if (iUserModel.type == LPConstants.LPUserType.Assistant) {
                    ass.add(iUserModel)
                }
                if (iUserModel.type == LPConstants.LPUserType.Student) {
                    stu.add(iUserModel)
                }
            }
        }
        ass.addAll(stu)
        return ass
    }

    private fun isApplicantAlreadyActive(activeList: List<IUserModel>): Boolean {
        for (a in activeList) {
            if (a.userId == userAdd.iUserModel.userId) {
                return true
            }
        }
        return false
    }

    override fun onDestroy() {
        super.onDestroy()
        RxUtils.dispose(disposeTimer)
        presenter.destroy()
    }

    override fun hideUIMenu(): Boolean {
        return false
    }

    override fun setWindowParams(windowParams: WindowManager.LayoutParams) {
        super.setWindowParams(windowParams)
        context?.let {
            windowParams.width = it.resources.getDimension(R.dimen.bjy_group_full_and_replace_width).toInt()
            windowParams.height = it.resources.getDimension(R.dimen.bjy_group_full_and_replace_height).toInt()
            windowParams.gravity = Gravity.CENTER
        }
    }

    override fun notifyActiveUserDataChange() {
        // 有人上下台、离开教室等
        if (isVisible) {
            remindActiveUserChange()
        }
    }

    override fun notifyUserDataChange() {
        if (isVisible) {
             replaceListRecyclerViewAdapter?.notifyCurrentActiveUserChange(filterTeacherFromActiveUserList())
        }
    }
}