package com.baijiayun.groupclassui.dialog;

import static com.baijiayun.bjyutils.kt.UtilsKt.getEncodePhoneNumber;

import android.annotation.SuppressLint;
import android.content.Context;
import android.graphics.Color;
import android.graphics.drawable.ColorDrawable;
import android.text.SpannableStringBuilder;
import android.text.Spanned;
import android.text.style.ForegroundColorSpan;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.PopupWindow;
import android.widget.TextView;

import com.baijiayun.bjyutils.drawable.DrawableBuilder;
import com.baijiayun.groupclassui.R;
import com.baijiayun.livebase.utils.ThemeDataUtil;
import com.baijiayun.liveuibase.utils.ShadowUtil;

import java.util.ArrayList;
import java.util.List;

public class HandsUpRemindPopupWindow extends PopupWindow {
    private Builder.IOnChangeText onChangeText;

    private HandsUpRemindPopupWindow(View contentView, int width, int height, boolean focusable) {
        super(contentView, width, height, focusable);
        setAnimationStyle(R.style.BJYPushInOutFromLeftBottom);
    }

    public static class Builder{
        private Context context;
        private HandsUpRemindPopupWindow popupWindow;
        private TextView remindText;
        private List<String> handsUpNames;
        private int lastCount;

        public Builder(Context context){
            this.context = context;
            handsUpNames = new ArrayList<>();
        }

        @SuppressLint("SetTextI18n")
        public Builder setHandsUpCount(int count, String name){
            if (count == 0) {
                handsUpNames.clear();
            }
            if (count < lastCount){
                handsUpNames.remove(name);
            }
            if (count > lastCount){
                handsUpNames.add(0,name);
            }
            lastCount = count;

            name = (handsUpNames.size() > 0 ? handsUpNames.get(0) : name);
            if (count == 1){
                remindText.setText(getEncodePhoneNumber(name) + context.getString(R.string.raising_hand));
            }else {
                remindText.setText(count + context.getString(R.string.students_raising_hand));
            }
            SpannableStringBuilder stringBuilder=new SpannableStringBuilder(remindText.getText());
            ForegroundColorSpan foregroundColorSpan=new ForegroundColorSpan(Color.parseColor("#FFAE00"));
            stringBuilder.setSpan(foregroundColorSpan,0
                    , count == 1 ? name.length() : (count > 10 ? 2 : 1)
                    , Spanned.SPAN_INCLUSIVE_EXCLUSIVE);
            remindText.setText(stringBuilder);
            return this;
        }

        public Builder build(){
            View handsUpPanel = LayoutInflater.from(context)
                    .inflate(R.layout.view_interactive_hands_up_remind, null);
            ShadowUtil.setViewBoundShadow(handsUpPanel.findViewById(R.id.bjysc_bottom_hands_up_shadow_container));
            handsUpPanel.findViewById(R.id.bjysc_bottom_hands_up_main_container).setBackground( new DrawableBuilder()
                    .solidColor(ThemeDataUtil.getColorFromThemeConfigByAttrId(
                            context, R.attr.base_theme_brand_container_color))
                    .cornerRadius(context.getResources().getDimensionPixelSize(R.dimen.bjysc_toolbar_bg_radius))
                    .build());
            remindText = handsUpPanel.findViewById(R.id.bjysc_bottom_hands_up_remind_text);
            popupWindow = new HandsUpRemindPopupWindow(handsUpPanel,
                    ViewGroup.LayoutParams.WRAP_CONTENT, ViewGroup.LayoutParams.WRAP_CONTENT, true);
            popupWindow.setBackgroundDrawable(new ColorDrawable(Color.TRANSPARENT));
            // 设置PopupWindow是否能响应外部点击事件
            popupWindow.setOutsideTouchable(true);
            popupWindow.setTouchable(true);
            popupWindow.setOnChangeText(this::setHandsUpCount);
            return this;
        }

        public HandsUpRemindPopupWindow getPopupWindow(){
            return popupWindow;
        }

        interface IOnChangeText {
            void changeRemindText(int count, String name);
        }
    }

    @Override
    public void dismiss() {
        super.dismiss();
        onChangeText.changeRemindText(0, "");
    }

    private void setOnChangeText(Builder.IOnChangeText onChangeText){
        this.onChangeText = onChangeText;
    }

    public void setRemindText(int count, String name){
        if (count >= 0){
            onChangeText.changeRemindText(count, name);
        }else {
            dismiss();
        }
    }
}
