package com.baijiayun.groupclassui.dialog

import android.content.Context
import android.os.Bundle
import android.view.Gravity
import android.view.View
import android.view.WindowManager
import androidx.core.content.ContextCompat
import com.baijiayun.bjyutils.drawable.DrawableBuilder
import com.baijiayun.bjyutils.kt.dp
import com.baijiayun.bjyutils.reactivex.RxUtils
import com.baijiayun.bjyutils.screen.DisplayUtils.getScreenHeightPixels
import com.baijiayun.bjyutils.screen.DisplayUtils.setRoundCorner
import com.baijiayun.groupclassui.R
import com.baijiayun.groupclassui.base.BaseGroupDialogFragment
import com.baijiayun.groupclassui.global.EventKey
import com.baijiayun.groupclassui.global.RouterListener
import com.baijiayun.groupclassui.util.convertTime
import com.baijiayun.livecore.models.LPStudyUserStatus
import com.baijiayun.liveuibase.base.BaseAdapter
import com.baijiayun.liveuibase.base.ViewHolder
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.disposables.Disposable
import kotlinx.android.synthetic.main.bjysc_dialog_study_rank.ivClose
import kotlinx.android.synthetic.main.bjysc_dialog_study_rank.layoutEmpty
import kotlinx.android.synthetic.main.bjysc_dialog_study_rank.listView
import kotlinx.android.synthetic.main.bjysc_item_study_rank.view.tvRank
import kotlinx.android.synthetic.main.bjysc_item_study_rank.view.tvStudyDuration
import kotlinx.android.synthetic.main.bjysc_item_study_rank.view.tvStudyStatus
import kotlinx.android.synthetic.main.bjysc_item_study_rank.view.tvStudyUserName

/**
 * Created by pengwenjian on 2020/12/10
 * Describe: 学生挂机dialog
 */
class StudyRankDialog : BaseGroupDialogFragment() {
    private var lpStudyTimeRanks: List<LPStudyUserStatus>? = null
    private var disposableOfTimeRank: Disposable? = null
    private var studyRankAdapter: StudyRankAdapter? = null
    override fun getLayoutId() = R.layout.bjysc_dialog_study_rank

    override fun init(savedInstanceState: Bundle?, arguments: Bundle?) {
        // do nothing
    }

    fun setStudyRankList(lpStudyTimeRanks: List<LPStudyUserStatus>) {
        this.lpStudyTimeRanks = lpStudyTimeRanks
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        if (context is RouterListener) {
            disposableOfTimeRank =
                (context as RouterListener).router.getSubjectByKey<List<LPStudyUserStatus>>(EventKey.StudyRankList)
                    .observeOn(AndroidSchedulers.mainThread())
                    .subscribe {
                        if (it.isNullOrEmpty()) {
                            layoutEmpty.visibility = View.VISIBLE
                        } else {
                            layoutEmpty.visibility = View.GONE
                            setStudyRankList(it)
                            studyRankAdapter?.setNewData(lpStudyTimeRanks)
                        }
                    }
        }
        ivClose.setOnClickListener { dismissAllowingStateLoss() }
        if (!lpStudyTimeRanks.isNullOrEmpty()) {
            layoutEmpty.visibility = View.GONE
            listView.divider = DrawableBuilder()
                .solidColor(ContextCompat.getColor(view.context, R.color.base_divider_line_other))
                .build()
            listView.dividerHeight = 1.dp
            studyRankAdapter = StudyRankAdapter(context)
            listView.adapter = studyRankAdapter
        } else {
            layoutEmpty.visibility = View.VISIBLE
        }
        studyRankAdapter?.setNewData(lpStudyTimeRanks)
    }

    override fun onStart() {
        super.onStart()
        dialog?.setCanceledOnTouchOutside(false)
        view?.run {
            setRoundCorner(this, 4f.dp)
        }
    }

    override fun setWindowParams(windowParams: WindowManager.LayoutParams) {
        super.setWindowParams(windowParams)
        windowParams.height =
            ((context?.let { getScreenHeightPixels(it) } ?: 0) * 330f / 376).toInt()
        windowParams.width = 320.dp
        windowParams.gravity = Gravity.CENTER
    }

    override fun onDestroyView() {
        super.onDestroyView()
        RxUtils.dispose(disposableOfTimeRank)
    }

    class StudyRankAdapter(context: Context?) :
        BaseAdapter<LPStudyUserStatus>(context, R.layout.bjysc_item_study_rank, null) {
        override fun convert(helper: ViewHolder, item: LPStudyUserStatus, position: Int) {
            val status = if (item.isLeave == 1 || item.isHangUp == 1) {
                helper.convertView.tvStudyStatus.isEnabled = false
                helper.convertView.tvStudyDuration.isEnabled = false
                mContext.getString(R.string.bjysc_study_room_study_seat_leave)
            } else {
                helper.convertView.tvStudyStatus.isEnabled = true
                helper.convertView.tvStudyDuration.isEnabled = true
                if (item.isActive == 1) {
                    if (item.isTutor == 1) {
                        mContext.getString(R.string.bjysc_study_room_tutor_status_helping)
                    } else {
                        mContext.getString(R.string.bjysc_study_room_study_seat_up)
                    }

                } else {
                    mContext.getString(R.string.bjysc_study_room_study_seat_down)
                }
            }
            helper.convertView.tvStudyStatus.text = status
            helper.convertView.tvStudyUserName.text = item.userName
            helper.convertView.tvStudyDuration.text = convertTime(item.duration)
            when (item.rank) {
                1 -> {
                    helper.convertView.tvRank.setBackgroundResource(R.drawable.uibase_ic_study_rank_1)
                    helper.convertView.tvRank.text = ""
                }
                2 -> {
                    helper.convertView.tvRank.setBackgroundResource(R.drawable.uibase_ic_study_rank_2)
                    helper.convertView.tvRank.text = ""
                }
                3 -> {
                    helper.convertView.tvRank.setBackgroundResource(R.drawable.uibase_ic_study_rank_3)
                    helper.convertView.tvRank.text = ""
                }
                else -> {
                    helper.convertView.tvRank.background = null
                    helper.convertView.tvRank.text = item.rank.toString()
                }
            }
        }
    }
}