package com.baijiayun.groupclassui.dialog

import android.os.Bundle
import android.view.View
import com.baijiayun.bjyutils.kt.dp
import com.baijiayun.bjyutils.reactivex.RxUtils
import com.baijiayun.livebase.widgets.dialog.NoteDialog
import com.baijiayun.livecore.listener.OnCountDownListener
import io.reactivex.Observable
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.disposables.Disposable
import java.util.concurrent.TimeUnit

/**
 * Created by pengwenjian on 2021/3/12
 * Describe: 带倒计时的noteDialog
 */
class TimerNoteDialog : NoteDialog() {
    private var positiveTextWidth = -1
    private var counter = 5L
    private var disposableOfTimer: Disposable? = null
    var countDownListener: OnCountDownListener? = null
    var hasTimer = false

    init {
        hideNegativeButton = true
        hideCheckBox = true
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)

        if (positiveTextWidth != -1) {
            tvPositive.width = positiveTextWidth
        } else {
            if (hasTimer) {
                tvPositive.width = 140.dp
            } else {
                tvPositive.width = 120.dp
            }
        }
        if (hasTimer) {
            disposableOfTimer = Observable.interval(0, 1, TimeUnit.SECONDS)
                .observeOn(AndroidSchedulers.mainThread())
                .take((counter + 1))
                .subscribe {
                    countDownListener?.onTimeCountDown(it.toInt(), counter.toInt())
                    if (it == counter) {
                        countDownListener?.onTimeOut()
                    }
                }
        }
    }

    fun setTimerCount(counter: Long) {
        this.counter = counter
    }

    override fun onDestroyView() {
        super.onDestroyView()
        RxUtils.dispose(disposableOfTimer)
    }
}