package com.baijiayun.groupclassui.global;

import androidx.annotation.VisibleForTesting;

import com.baijiayun.livecore.context.LiveRoom;
import com.baijiayun.liveuibase.widgets.toolbar.BaseGraphMenuWindow;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.ConcurrentLinkedQueue;

import io.reactivex.subjects.BehaviorSubject;
import io.reactivex.subjects.PublishSubject;
import io.reactivex.subjects.Subject;

/**
 * Created by Shubo on 2018/9/12.
 */
public class RouterImpl implements IRouter {
    //粘性事件，BehaviorSubject
    private HashMap<EventKey, Subject<?>> routerMap;
    //普通事件，PublicSubject
    private HashMap<EventCode, Subject<?>> publishMap;
    //存储值
    private HashMap<EventKey, Object> routerStore;

    private LiveRoom liveRoom;

    RouterImpl(LiveRoom liveRoom) {
        this.liveRoom = liveRoom;
        routerMap = new HashMap<>();
        publishMap = new HashMap<>();
        routerStore = new HashMap<>();
    }

    /**
     * 通过key获取Subject
     *
     * @param key
     * @return
     */
    @Override
    public <T> BehaviorSubject<T> getSubjectByKey(EventKey key) {
        BehaviorSubject<T> publishSubject = (BehaviorSubject<T>) routerMap.get(key);
        if(publishSubject == null){
            publishSubject = BehaviorSubject.create();
            routerMap.put(key, publishSubject);
        }
        return publishSubject;
    }

    @Override
    public <T> PublishSubject<T> getPublishSubjectByKey(EventCode key) {
        PublishSubject<T> publishSubject = (PublishSubject<T>) publishMap.get(key);
        if(publishSubject == null){
            publishSubject = PublishSubject.create();
            publishMap.put(key, publishSubject);
        }
        return publishSubject;
    }

    @Override
    public <T> T getValueByKey(EventKey key, Class<T> targetClass) {
        return objParse(routerStore.get(key), targetClass);
    }

    @Override
    public <T> T getValueByKey(EventKey key, Class<T> targetClass, T defaultValue) {
        T rst = getValueByKey(key, targetClass);
        return rst == null ? defaultValue : rst;
    }

    @Override
    public <K, V> Map<K, V> getMapValueByKey(EventKey eventKey, Class<K> keyClass, Class<V> valueClass) {
        Map<?, ?> tmpRst = getValueByKey(eventKey, Map.class);
        if (tmpRst != null) {
            Map<K, V> rst = new HashMap<>();
            for (Map.Entry<?, ?> entry : tmpRst.entrySet()) {
                K key = objParse(entry.getKey(), keyClass);
                V value = objParse(entry.getValue(), valueClass);
                if (key != null && value != null) {
                    rst.put(key, value);
                }
            }
            return rst;
        }
        return new HashMap<>();
    }

    @Override
    public <T> List<T> getListByKey(EventKey key, Class<T> targetClass) {
        List<?> value = getValueByKey(key, ArrayList.class);
        if (value != null) {
            List<T> rst = new ArrayList<>();
            for (Object item : value) {
                T target = objParse(item, targetClass);
                if (target != null) {
                    rst.add(target);
                }
            }
            return rst;
        }
        return new ArrayList<>();
    }

    @Override
    public <T> Set<T> getSetByKey(EventKey key, Class<T> targetClass) {
        Set<?> value = getValueByKey(key, HashSet.class);
        if (value != null) {
            Set<T> rst = new HashSet<>();
            for (Object item : value) {
                T target = objParse(item, targetClass);
                if (target != null) {
                    rst.add(target);
                }
            }
            return rst;
        }
        return new HashSet<>();
    }

    private  <T> T objParse(Object value, Class<T> targetClass) {
        try {
            if (targetClass.isInstance(value)) {
                return targetClass.cast(value);
            }
        } catch (Exception ignored) {
            // do nothing
        }
        return null;
    }

    @Override
    public void setObjectByKey(EventKey key, Object object) {
        routerStore.put(key, object);
    }

    @Override
    public <T> ConcurrentLinkedQueue<T> getQueueByKey(EventKey key, Class<T> targetClass) {
        ConcurrentLinkedQueue<?> value = getValueByKey(key, ConcurrentLinkedQueue.class);
        if (value != null) {
            ConcurrentLinkedQueue<T> rst = new ConcurrentLinkedQueue<>();
            for (Object item : value) {
                T target = objParse(item, targetClass);
                if (target != null) {
                    rst.add(target);
                }
            }
            return rst;
        }
        return new ConcurrentLinkedQueue<>();
    }

    @Override
    public <T> T getSubjectValueByKey(EventKey key, Class<T> targetClass) {
        final Subject<?> subject = routerMap.get(key);
        if (subject == null) {
            return null;
        }
        BehaviorSubject<?> behaviorSubject = (BehaviorSubject<?>) subject;
        return objParse(behaviorSubject.getValue(), targetClass);
    }

    @Override
    public <T> T getSubjectValueByKey(EventKey key, Class<T> targetClass, T defaultValue) {
        final T parseValue = getSubjectValueByKey(key, targetClass);
        return parseValue == null ? defaultValue : parseValue;
    }

    @Override
    public LiveRoom getLiveRoom() {
        return liveRoom;
    }

    @Override
    public void setLiveRoom(LiveRoom liveRoom) {
        this.liveRoom = liveRoom;
    }

    public void release() {
        if(liveRoom != null){
            liveRoom.quitRoom();
            liveRoom = null;
        }
        routerMap.clear();
        publishMap.clear();
        routerStore.clear();
    }

    public void clear(){
        routerStore.clear();
        publishMap.clear();
        final Iterator<Map.Entry<EventKey, Subject<?>>> iterator = routerMap.entrySet().iterator();
        while (iterator.hasNext()) {
            final Map.Entry<EventKey, Subject<?>> next = iterator.next();
            if (next.getKey() != EventKey.ReminderMessage && next.getKey() != EventKey.ReminderErrorMessage
                    && next.getKey() != EventKey.ReminderBlackboardPage && next.getKey() != EventKey.GraphDrawEnable) {
                iterator.remove();
            }
        }
        //TODO toolbarWindow用同一个，不要这些奇怪逻辑
        //断线重连后图形画笔记住状态，清空点击状态
        final Object value = getSubjectByKey(EventKey.GraphDrawEnable).getValue();
        if (value instanceof BaseGraphMenuWindow.OnShapeChangeModel) {
            BaseGraphMenuWindow.OnShapeChangeModel model = (BaseGraphMenuWindow.OnShapeChangeModel) value;
            model.setInitDrawable(true);
        }
    }

    @VisibleForTesting
    public static void main(String[] args) {
        // test
    }
}
