package com.baijiayun.groupclassui.layer;

import android.annotation.SuppressLint;
import android.content.Context;
import android.os.Build;
import android.util.AttributeSet;
import android.util.Log;
import android.view.ViewGroup;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.annotation.RequiresApi;

import com.baijiayun.bjyutils.reactivex.RxUtils;
import com.baijiayun.groupclassui.R;
import com.baijiayun.groupclassui.global.EventKey;
import com.baijiayun.groupclassui.window.group.GroupManageWindow;
import com.baijiayun.groupclassui.window.group.SetGroupCountWindow;
import com.baijiayun.livebase.widgets.dialog.CommonDialog;
import com.baijiayun.livecore.models.LPDivideGroupModel;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import io.reactivex.disposables.Disposable;

/**
 * 分组layer
 * Created by dujuncan on 2018/09/28.
 */
public class GroupLayer extends BaseLayerOfProfessionalClass {

    private final String TAG = GroupLayer.class.getCanonicalName();

    private SetGroupCountWindow setGroupCountWindow;
    private GroupManageWindow groupManageWindow;
    private CommonDialog dissolveDialog;

    private List<Disposable> disposables;

    public GroupLayer(@NonNull Context context) {
        super(context);
    }

    public GroupLayer(@NonNull Context context, @Nullable AttributeSet attrs) {
        super(context, attrs);
    }

    public GroupLayer(@NonNull Context context, @Nullable AttributeSet attrs, int defStyleAttr) {
        super(context, attrs, defStyleAttr);
    }

    @RequiresApi(api = Build.VERSION_CODES.LOLLIPOP)
    public GroupLayer(@NonNull Context context, @Nullable AttributeSet attrs, int defStyleAttr, int defStyleRes) {
        super(context, attrs, defStyleAttr, defStyleRes);
    }

    @SuppressLint("ClickableViewAccessibility")
    @Override
    protected void init() {
        super.init();
        Disposable disposableDisplayAllDissolve = router.getSubjectByKey(EventKey.DisplayWindowAllDissolve)
                .ofType(Boolean.class)
                .filter(aBoolean -> aBoolean)
                .subscribe(aBoolean -> {
                    Log.d(TAG, "DisplayWindowAllDissolve: " + aBoolean);
                    showAllDissolve();
                });
        Disposable disposableDissolveAllEnable = router.getSubjectByKey(EventKey.GroupDissolveAllEnable)
                .ofType(Boolean.class)
                .subscribe(aBoolean -> {
                    Log.d(TAG, "GroupDissolveAllEnable: " + aBoolean);
                    if (aBoolean) {
                        //解散全部
                        router.getSubjectByKey(EventKey.GroupManageLayout)
                                .onNext(false);
                        LPDivideGroupModel model = new LPDivideGroupModel();
                        model.groups = new HashMap<>();
                        router.getLiveRoom().requestDivideGroup(model);
                    } else {
                        if (groupManageWindow != null)
                            groupManageWindow.getView().setVisibility(VISIBLE);
                    }
                });
        disposables = new ArrayList<>();
        disposables.add(disposableDisplayAllDissolve);
        disposables.add(disposableDissolveAllEnable);

        showSetGroupCount();
    }

    private void showSetGroupCount() {
        setGroupCountWindow = new SetGroupCountWindow(getContext());
        setGroupCountWindow.setOnNextStepListener(groupCount -> GroupLayer.this.showGroupManage(groupCount));
    }

    private void showGroupManage(int groupCount) {
        if (setGroupCountWindow != null) removeWindow(setGroupCountWindow);
        LayoutParams params = new LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.MATCH_PARENT);
        groupManageWindow = new GroupManageWindow(getContext(), groupCount);
        addWindow(groupManageWindow, params);
    }

    private void showAllDissolve() {
        if (groupManageWindow != null) groupManageWindow.getView().setVisibility(INVISIBLE);
        if (dissolveDialog == null) {
            dissolveDialog = new CommonDialog(getContext(), CommonDialog.ChoiceMode.Double_Red)
                    .setMainDisplayText(getContext().getResources().getString(R.string.bjysc_group_all_dissolve_tips))
                    .setNegative(getContext().getResources().getString(R.string.bjysc_cancel), (dialog, which) -> {
                        router.getSubjectByKey(EventKey.GroupDissolveAllEnable).onNext(false);
                        dialog.dismiss();
                    })
                    .setPositive(getContext().getResources().getString(R.string.bjysc_cancel), (dialog, which) -> {
                        router.getSubjectByKey(EventKey.GroupDissolveAllEnable).onNext(true);
                        dialog.dismiss();
                    });
        }
        dissolveDialog.show();
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
        if (disposables != null) {
            for (Disposable disposable : disposables) {
                RxUtils.dispose(disposable);
            }
            disposables.clear();
            disposables = null;
        }
    }
}
