package com.baijiayun.groupclassui.layer

import android.content.Context
import android.text.TextUtils
import android.util.AttributeSet
import androidx.fragment.app.FragmentActivity
import com.baijiayun.groupclassui.R
import com.baijiayun.groupclassui.container.VideoContainerListener
import com.baijiayun.groupclassui.dialog.QRcodeDialog
import com.baijiayun.groupclassui.global.EventKey
import com.baijiayun.groupclassui.util.LayoutUtils
import com.baijiayun.groupclassui.window.video.VideoContainerWindow
import com.baijiayun.livebase.widgets.dialog.NoteDialog
import com.baijiayun.livebase.context.LPConstants
import com.baijiayun.livecore.models.imodels.IMediaModel
import com.baijiayun.livecore.models.roomresponse.LPResRoomUserInModel
import kotlin.math.ceil

/**
 * Created by pengwenjian on 2020/12/4
 * Describe: 自习室逻辑
 * 1.进教室任何人检查未上课即开始上课
 */
class StudyRoomLayer : SeatLayer {
    constructor(ctx: Context) : super(ctx)
    constructor(ctx: Context, attributeSet: AttributeSet?) : super(ctx, attributeSet)
    constructor(ctx: Context, attributeSet: AttributeSet?, defStyleAttr: Int) : super(ctx, attributeSet, defStyleAttr)
    private var noteDialog: NoteDialog? = null
    var lisenter: VideoContainerListener? = null

    override fun showTeacherPlaceholder() = super.showTeacherPlaceholder() && router.liveRoom.studyRoomVM.studyRoomMode == LPConstants.StudyRoomMode.Tutor

    fun switchStudyRoomMode(roomMode: LPConstants.StudyRoomMode) {
        studyRoomMode = roomMode
        if (studyRoomMode.isGalleryLayout) {
            removeEmptyView()
        } else {
            if (!hasTeacherSpeaker()) {
                addEmptyView()
            }
        }
        switchLayoutMode(!roomMode.isGalleryLayout)
    }

    override fun addEmptyView() {
        //助教切换教室后先发送的切换信令，后发送的重进这里没有时序关系，需要判断
        if (router == null || mLiveRoom == null) {
            return
        }
        super.addEmptyView()
    }
    private fun hasTeacherSpeaker():Boolean{
        if(mSpeakList.isNullOrEmpty()){
            return false
        }
        for (mediaModel in mSpeakList) {
            if (mediaModel.user.type == LPConstants.LPUserType.Teacher) {
                return true
            }
        }
        return false
    }

    override fun handsAutoAttachAudio(): Boolean {
        return router.liveRoom.studyRoomVM.studyRoomMode != LPConstants.StudyRoomMode.SelfStudy
    }

    override fun isBoardLayout() = !studyRoomMode.isGalleryLayout

    override fun onActiveUserDeny(lpResRoomUserInModel: LPResRoomUserInModel) {
        if (!TextUtils.equals(router.liveRoom.currentUser.userId, lpResRoomUserInModel.user.userId)) {
            super.onActiveUserDeny(lpResRoomUserInModel)
            return
        }
        if (context is FragmentActivity) {
            (context as FragmentActivity).let {
                if (noteDialog?.isAdded == true) {
                    noteDialog?.dismissAllowingStateLoss()
                }
                noteDialog = NoteDialog()
                noteDialog?.run {
                    mainText = it.getString(R.string.bjysc_study_room_grab_seat_fail_main_tip)
                    assistantText = it.getString(R.string.bjysc_study_room_grab_seat_fail_assistant_tip)
                    hideCheckBox = true
                    hideNegativeButton = true
                    context?.let {
                        width = it.resources.getDimensionPixelSize(R.dimen.bjy_base_common_dialog_container_width)
                        height = it.resources.getDimensionPixelSize(R.dimen.bjy_base_common_dialog_container_height)
                    }
                    show(it.supportFragmentManager, QRcodeDialog::class.java.simpleName)
                }
            }
        }
    }
    //不自动上台
    override fun autoOnStage(): Boolean {
        return false
    }

    override fun addVideo(iMediaModel: IMediaModel, isPlay: Boolean) {
        super.addVideo(iMediaModel, isPlay)
        //自己的视频或者超出人数则让按钮消失
        if (TextUtils.equals(iMediaModel.user.number, mLiveRoom.currentUser.number) || mSpeakVideos.size >= maxSpeakers) {
            router.getSubjectByKey<Any>(EventKey.NoteSelfActive).onNext(true)
        }
    }

    override fun removeVideo(iMediaModel: IMediaModel) {
        super.removeVideo(iMediaModel)
        if (mSpeakVideos.size < maxSpeakers && !mSpeakVideos.containsKey(mLiveRoom.currentUser.userId)) {
            router.getSubjectByKey<Any>(EventKey.NoteSelfActive).onNext(false)
        }
    }

    override fun onMeasure(widthMeasureSpec: Int, heightMeasureSpec: Int) {
        val selfWidth = MeasureSpec.getSize(widthMeasureSpec)
        val selfHeight = MeasureSpec.getSize(heightMeasureSpec)
        setMeasuredDimension(selfWidth, selfHeight)

        if (maxSpeakers <= 0) {
            return
        }

        var childCount = mSpeakVideos.size
        val childWidth: Int
        val childHeight: Int
        if (!studyRoomMode.isGalleryLayout) {
            //板书布局，水平居中排列
            //学生不满6个，布局居中,宽度按6等分计算。否则按比例平分铺满
            //因为还有一个老师，所以是7
            val ration = 16 / 9.0f
            val maxEqualWindow = 6
            if (hasPlaceHolder) {
                childCount++
            }
            childHeight = selfHeight
            childWidth = if (childCount <= maxEqualWindow) {
                (childHeight * ration).toInt()
            } else {
                selfWidth / childCount
            }
        } else {
            childWidth = selfWidth / LayoutUtils.getMaxNumPerLine(maxSpeakers)
            childHeight = selfHeight / LayoutUtils.getLineCount(maxSpeakers)
        }
        val childWidthMeasureSpec = MeasureSpec.makeMeasureSpec(childWidth, MeasureSpec.EXACTLY)
        val childHeightMeasureSpec = MeasureSpec.makeMeasureSpec(childHeight, MeasureSpec.EXACTLY)
        for (i in 0 until childCount) {
            val childAt = getChildAt(i)
            childAt.measure(childWidthMeasureSpec, childHeightMeasureSpec)
        }
    }

    override fun onLayout(changed: Boolean, left: Int, top: Int, right: Int, bottom: Int) {
        //使用内部维护的视频列表。按顺序取，重新队列
        val placeHolderCount = if (hasPlaceHolder) 1 else 0
        var childCount = placeHolderCount
        val videoWindows: Array<Any> = mSpeakVideos.values.toTypedArray()
        for (videoWindow in videoWindows) {
            val isTeacherVideo =
                ((videoWindow as VideoContainerWindow).mediaModel.user.type == LPConstants.LPUserType.Teacher
                        || videoWindow.mediaModel.user.type == LPConstants.LPUserType.Assistant)
            val isLocalVideo = videoWindow.videoWindow.isLocalVideo
            if (!disableOtherStudentVideo || isTeacherVideo || isLocalVideo
                || router.liveRoom.isTeacherOrAssistant || !isBoardLayout
            ) {
                // 画廊布局全部显示；本地是老师或助教，全部显示；教室内未开启不看其他学生，全部显示；其他情况，仅展示本地视频和老师
                childCount++
            }
        }
        // endregion
        var hasAddCount = 0
        for (i in 0 until mSpeakVideos.size + placeHolderCount) {
            val childAt = getChildAt(i)
            var childLeft: Int
            var childRight: Int
            var childTop: Int
            var childBottom: Int
            if (!studyRoomMode.isGalleryLayout) {
                var hideThisWindow = false
                if (!(hasPlaceHolder && i == 0)) {
                    //使用内部维护的视频列表。按顺序取，重新队列
                    val isTeacherOrAssistantVideo =
                        (videoWindows[i - placeHolderCount] as VideoContainerWindow)
                            .mediaModel.user.type == LPConstants.LPUserType.Teacher ||
                                (videoWindows.get(i - placeHolderCount) as VideoContainerWindow)
                                    .mediaModel.user.type == LPConstants.LPUserType.Assistant
                    val isLocalVideo =
                        (videoWindows[i - placeHolderCount] as VideoContainerWindow)
                            .videoWindow.isLocalVideo
                    if (disableOtherStudentVideo && !isTeacherOrAssistantVideo && !isLocalVideo) {
                        hideThisWindow = true
                    }
                }
                // 支持不看其他学生
                if (hideThisWindow) {
                    childLeft = 0
                    childRight = 0
                    childTop = 0
                    childBottom = 0
                } else {
                    childLeft = ((measuredWidth - childAt.measuredWidth * childCount) / 2
                            + childAt.measuredWidth * hasAddCount++)
                    childRight = childLeft + childAt.measuredWidth
                    childTop = top
                    childBottom = bottom
                }
            } else {
                // 每行最多个数
                val maxNumPerLine = LayoutUtils.getMaxNumPerLine(maxSpeakers)
                // 当前行号
                val nowLine = i / maxNumPerLine
                // region 上下计算
                // 行高
                val itemHeight =
                    (measuredHeight / ceil(maxSpeakers.toDouble() / maxNumPerLine)).toInt()
                // region 特殊处理，只有两个时，上下也需要补齐空间
//                    itemHeight /= childCount == 2 ? 2 : 1;
//                    int firstLineOffset = childCount == 2 ? itemHeight / 2 : 0;
                // endregion
                // childTop = top + 前几行的高度 + 首行偏移
                childTop = top + nowLine * itemHeight
                // region 非最后一行，加一像素空隙
                val isLastLine = (nowLine + 1) * maxNumPerLine >= maxSpeakers
                childBottom = childTop + itemHeight - if (isLastLine) 0 else 1
                // endregion
                // endregion

                // region 左右计算
                // 每个宽度
                val itemWidth = measuredWidth / maxNumPerLine
                // 当前行的个数
                val nowLineMaxNum =
                    (maxSpeakers - maxNumPerLine * nowLine).coerceAtMost(maxNumPerLine)
                // 根据当前行个数，和每行最多个数计算当前行首个的左偏移
                val nowLineLeftOffset = (maxNumPerLine - nowLineMaxNum) * itemWidth / 2

                // childLeft = 行左偏移 + 前几个的宽度
                childLeft = nowLineLeftOffset + (i - maxNumPerLine * nowLine) * itemWidth

                // region 非当前行最后一个，加一像素空隙
                val isLastItemInNowLine = (i + 1) / maxNumPerLine != nowLine
                childRight = childLeft + itemWidth - if (isLastItemInNowLine) 0 else 1
                // endregion
                // endregion
            }
            childAt.layout(childLeft, childTop, childRight, childBottom)
        }
        //每次用户发生变化时，通知视频菜单.
        router.getSubjectByKey<Any>(EventKey.RepositionVideoMenu).onNext(0)
    }

    override fun onDestroy() {
        super.onDestroy()
        noteDialog?.dismiss()
    }
}