package com.baijiayun.groupclassui.layer

import android.content.Context
import android.util.AttributeSet
import com.baijiayun.groupclassui.util.LayoutUtils
import kotlin.math.ceil

/**
 * Created by pengwenjian on 2020/12/4
 * Describe: 占位图
 */
class StudyRoomPlaceholderLayer :
    BaseLayerOfProfessionalClass {
    private var maxSpeakers = 0

    constructor(ctx: Context) : super(ctx)
    constructor(ctx: Context, attributeSet: AttributeSet?) : super(ctx, attributeSet)
    constructor(ctx: Context, attributeSet: AttributeSet?, defStyleAttr: Int) : super(ctx, attributeSet, defStyleAttr)

    override fun onRoomActive() {
        super.onRoomActive()
        maxSpeakers = router.liveRoom.maxActiveUsers
    }

    override fun onMeasure(widthMeasureSpec: Int, heightMeasureSpec: Int) {
        val selfWidth = MeasureSpec.getSize(widthMeasureSpec)
        val selfHeight = MeasureSpec.getSize(heightMeasureSpec)
        setMeasuredDimension(selfWidth, selfHeight)
        if (maxSpeakers <= 0) {
            return
        }
        val childWidth = selfWidth / LayoutUtils.getMaxNumPerLine(maxSpeakers)
        val childHeight = selfHeight / LayoutUtils.getLineCount(maxSpeakers)
        val childWidthMeasureSpec = MeasureSpec.makeMeasureSpec(childWidth, MeasureSpec.EXACTLY)
        val childHeightMeasureSpec = MeasureSpec.makeMeasureSpec(childHeight, MeasureSpec.EXACTLY)
        for (i in 0 until maxSpeakers) {
            val childAt = getChildAt(i)
            childAt.measure(childWidthMeasureSpec, childHeightMeasureSpec)
        }
    }

    override fun onLayout(changed: Boolean, left: Int, top: Int, right: Int, bottom: Int) {
        // endregion
        for (i in 0 until maxSpeakers) {
            val childAt = getChildAt(i)
            var childLeft: Int
            var childRight: Int
            var childTop: Int
            var childBottom: Int
            // 每行最多个数
            val maxNumPerLine = LayoutUtils.getMaxNumPerLine(maxSpeakers)
            // 当前行号
            val nowLine = i / maxNumPerLine
            // region 上下计算
            // 行高
            val itemHeight = (measuredHeight / ceil(maxSpeakers.toDouble() / maxNumPerLine)).toInt()
            // region 特殊处理，只有两个时，上下也需要补齐空间
//                    itemHeight /= childCount == 2 ? 2 : 1;
//                    int firstLineOffset = childCount == 2 ? itemHeight / 2 : 0;
            // endregion
            // childTop = top + 前几行的高度 + 首行偏移
            childTop = top + nowLine * itemHeight
            // region 非最后一行，加一像素空隙
            childBottom = childTop + itemHeight
            // endregion
            // endregion

            // region 左右计算
            // 每个宽度
            val itemWidth = measuredWidth / maxNumPerLine
            // 当前行的个数
            val nowLineMaxNum = (maxSpeakers - maxNumPerLine * nowLine).coerceAtMost(maxNumPerLine)
            // 根据当前行个数，和每行最多个数计算当前行首个的左偏移
            val nowLineLeftOffset = (maxNumPerLine - nowLineMaxNum) * itemWidth / 2

            // childLeft = 行左偏移 + 前几个的宽度
            childLeft = nowLineLeftOffset + (i - maxNumPerLine * nowLine) * itemWidth

            // region 非当前行最后一个，加一像素空隙
            childRight = childLeft + itemWidth
            // endregion
            // endregion
            childAt.layout(childLeft, childTop, childRight, childBottom)
        }
    }
}