package com.baijiayun.groupclassui.user;

import android.text.TextUtils;

import com.baijiayun.bjyutils.reactivex.RxUtils;
import com.baijiayun.groupclassui.base.BaseGroupPresenter;
import com.baijiayun.groupclassui.dialog.FullAndReplaceListener;
import com.baijiayun.groupclassui.global.EventCode;
import com.baijiayun.groupclassui.global.IRouter;
import com.baijiayun.livebase.context.LPConstants;
import com.baijiayun.livecore.models.imodels.IMediaControlModel;
import com.baijiayun.livebase.models.imodels.IUserModel;

import io.reactivex.android.schedulers.AndroidSchedulers;
import io.reactivex.disposables.CompositeDisposable;

/**
 * replaceWindow作用的Presenter
 *
 * @author chenfu
 */
public class ReplacePresenter implements BaseGroupPresenter {
    CompositeDisposable disposables;
    private IRouter router;
    private IUserInteractionListener listener;
    // 上次在线人数，不包括老师
    private int lastOnlineUserCount = -1;
    private int lastActiveUserCount = 0;
    private IMediaControlModel iMediaControlModel;

    public ReplacePresenter(IUserInteractionListener iUserInteractionListener, IRouter iRouter) {
        this.listener = (FullAndReplaceListener) iUserInteractionListener;
        this.router = iRouter;
    }

    @Override
    public void setRouter(IRouter router) {
        this.router = router;
    }

    public IRouter getRouter() {
        return router;
    }

    @Override
    public void subscribe() {
        disposables = new CompositeDisposable();
        disposables.add(router.getLiveRoom().getOnlineUserVM()
                .getObservableOfOnlineUser()
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(iUserModels -> {
                    IUserModel tea = null;
                    for (IUserModel iUserModel : iUserModels) {
                        if (iUserModel.getType() == LPConstants.LPUserType.Teacher) {
                            tea = iUserModel;
                            break;
                        }
                    }
                    // 过滤老师
                    iUserModels.remove(tea);
                    if (listener != null) {
                        if (lastOnlineUserCount > iUserModels.size()) {
                            // 有学生或助教离开教室，包括非申请人离开、申请人离开(包括举手时)
                            listener.notifyActiveUserDataChange();
                        } else if (lastOnlineUserCount == iUserModels.size()) {
                            // 举手取消和举手申请人离开的第二次onlineuser回调
                            listener.notifyActiveUserDataChange();
                        }
                    }
                    lastOnlineUserCount = iUserModels.size();
                }));

        // 初次创建回调时可能会回调触发上次上台操作的回调，但是activeUser不变
        disposables.add(router.getLiveRoom().getOnlineUserVM()
                .getPublishSubjectOfActiveUserAdd()
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(userModel -> {
                    if (listener != null && userModel.getUser().getType() != LPConstants.LPUserType.Teacher) {
                        int activeUserCount = router.getLiveRoom().getOnlineUserVM().getActiveUserList().size();
                        if (lastActiveUserCount < activeUserCount) {
                            listener.notifyActiveUserDataChange();
                            lastActiveUserCount = activeUserCount;
                        }
                    }
                }));
        disposables.add(router.getLiveRoom().getOnlineUserVM()
                .getPublishSubjectOfActiveUserRemove()
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(userModel -> {
                    if (listener != null && userModel.getUser().getType() != LPConstants.LPUserType.Teacher) {
                        int activeUserCount = router.getLiveRoom().getOnlineUserVM().getActiveUserList().size();
                        if (lastActiveUserCount > activeUserCount) {
                            listener.notifyActiveUserDataChange();
                            lastActiveUserCount = activeUserCount;
                        }
                    }
                }));
        disposables.add(router.getLiveRoom().getSpeakQueueVM()
                .getObservableOfSpeakResponse()
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(iMediaControlModel -> {
                    if (!iMediaControlModel.isApplyAgreed()) {
                        // 由于取消举手和举手中离开无法通过此回调分开，因此需要主动触发onlineuser回调
                        router.getLiveRoom().getOnlineUserVM().loadMoreUser();
                    }
                }));
        // 其他人开关音视频
        disposables.add(router.getLiveRoom().getSpeakQueueVM()
                .getObservableOfMediaPublish()
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(iMediaModel -> {
                    int curActiveCount = router.getLiveRoom().getOnlineUserVM().getActiveUserList().size();
                    if (lastActiveUserCount < curActiveCount) {
                        // 当被其他人同意的举手申请人成功上台发言时
                        listener.notifyActiveUserDataChange();
                    }
                    lastActiveUserCount = curActiveCount;
                    if (iMediaModel.getUser().getType() != LPConstants.LPUserType.Teacher) {
                        listener.notifyUserDataChange();
                    }
                }));
        // 助教自己被其他人开关音视频
        disposables.add(router.getLiveRoom().getSpeakQueueVM()
                .getObservableOfMediaControl()
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(iMediaControlModel -> {
                    if (iMediaControlModel.getUser().getUserId().equals(router.getLiveRoom().getCurrentUser().getUserId())) {
                        this.iMediaControlModel = iMediaControlModel;
                    }
                }));

        disposables.add(router.getPublishSubjectByKey(EventCode.AttachVideo)
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(o -> {
                    if (iMediaControlModel != null
                            && iMediaControlModel.getUser().getUserId().equals(router.getLiveRoom().getCurrentUser().getUserId())) {
                        if (listener != null) {
                            listener.notifyUserDataChange();
                        }
                    }
                }));
        disposables.add(router.getPublishSubjectByKey(EventCode.AttachAudio)
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(o -> {
                    if (iMediaControlModel != null
                            && iMediaControlModel.getUser().getUserId().equals(router.getLiveRoom().getCurrentUser().getUserId())) {
                        if (listener != null) {
                            listener.notifyUserDataChange();
                        }
                    }
                }));
        // 初始化lastOnlineUserCount
        router.getLiveRoom().getOnlineUserVM().loadMoreUser();
        lastActiveUserCount = router.getLiveRoom().getOnlineUserVM().getActiveUserList().size();
    }

    @Override
    public void unSubscribe() {
        // do nothing
    }

    @Override
    public void destroy() {
        RxUtils.dispose(disposables);
    }

    /**
     * router.getLiveRoom().getRecorder().isVideoAttached()
     * 上述方法在所有activeUser都不发言的情况下，当自己是助教时，其他人远程控制自己都返回为false
     * 而当有人发言时，上述方法会返回为正确的状态
     * 可能是本地视频在判断时还未打开的原因
     *
     */
    public boolean isSelfVideoOn() {
        return router.getLiveRoom().getRecorder().isVideoAttached() || (iMediaControlModel != null
                && iMediaControlModel.getUser().getUserId().equals(router.getLiveRoom().getCurrentUser().getUserId())
                && iMediaControlModel.isVideoOn());
    }

    public boolean isSelfAudioOn() {
        return router.getLiveRoom().getRecorder().isAudioAttached() || (iMediaControlModel != null
                && iMediaControlModel.getUser().getUserId().equals(router.getLiveRoom().getCurrentUser().getUserId())
                && iMediaControlModel.isAudioOn());
    }

    public String getAssistantCustomLabelText() {
        return TextUtils.isEmpty(router.getLiveRoom().getCustomizeAssistantLabel()) ? "助教" : router.getLiveRoom().getCustomizeAssistantLabel();
    }
}
