package com.baijiayun.groupclassui.widget;

import android.annotation.SuppressLint;
import android.content.Context;
import androidx.core.view.GestureDetectorCompat;
import androidx.appcompat.widget.AppCompatImageView;
import android.util.AttributeSet;
import android.view.GestureDetector;
import android.view.MotionEvent;
import android.view.View;

/**
 * Created by Dujuncan on 2018/11/6
 */
public class LongPressImageView extends AppCompatImageView {

    private GestureDetectorCompat gestureDetectorCompat;
    private LongPressGesture longPressGesture;
    private OnLongPressListener onLongPressListener;

    public LongPressImageView(Context context) {
        this(context, null);
    }

    public LongPressImageView(Context context, AttributeSet attrs) {
        this(context, attrs, 0);
    }

    public LongPressImageView(Context context, AttributeSet attrs, int defStyleAttr) {
        super(context, attrs, defStyleAttr);
        init();
    }

    private void init() {
        longPressGesture = new LongPressGesture();
        gestureDetectorCompat = new GestureDetectorCompat(getContext(), longPressGesture);
        setOnClickListener(v -> {
        });
    }

    @SuppressLint("ClickableViewAccessibility")
    @Override
    public boolean onTouchEvent(MotionEvent event) {
        gestureDetectorCompat.onTouchEvent(event);
        if (event.getAction() == MotionEvent.ACTION_UP) {//举手隐藏，说明已经同意或者被远程打开，不用取消
            if (getVisibility() != VISIBLE) return super.onTouchEvent(event);
            if (longPressGesture.isLongPressed) {
                longPressGesture.isLongPressed = false;
                if (onLongPressListener != null) {
                    onLongPressListener.onCancelLongPress(LongPressImageView.this);
                }
            }
        }
        return super.onTouchEvent(event);
    }

    class LongPressGesture extends GestureDetector.SimpleOnGestureListener {
        boolean isLongPressed;

        @Override
        public boolean onSingleTapUp(MotionEvent e) {
            if (onLongPressListener != null) {
                onLongPressListener.onClick(LongPressImageView.this);
            }
            return true;
        }

        @Override
        public void onLongPress(MotionEvent e) {
            super.onLongPress(e);
            if (onLongPressListener != null) {
                onLongPressListener.onLongPress(LongPressImageView.this);
            }
            isLongPressed = true;
        }
    }

    public interface OnLongPressListener {
        void onClick(View view);

        void onLongPress(View view);

        void onCancelLongPress(View view);
    }

    public void setOnLongPressListener(OnLongPressListener onLongPressListener) {
        this.onLongPressListener = onLongPressListener;
    }
}