package com.baijiayun.groupclassui.window.blackboard;

import static android.view.View.OVER_SCROLL_NEVER;
import static android.view.ViewGroup.LayoutParams.MATCH_PARENT;

import android.content.Context;
import android.graphics.Bitmap;
import android.text.TextUtils;
import android.view.View;
import android.widget.LinearLayout;
import android.widget.ScrollView;

import com.baijiayun.bjyutils.reactivex.RxUtils;
import com.baijiayun.glide.Glide;
import com.baijiayun.glide.request.target.SimpleTarget;
import com.baijiayun.glide.request.transition.Transition;
import com.baijiayun.groupclassui.R;
import com.baijiayun.groupclassui.container.ShapePaint;
import com.baijiayun.groupclassui.global.EventKey;
import com.baijiayun.groupclassui.global.IRouter;
import com.baijiayun.groupclassui.global.RouterListener;
import com.baijiayun.livebase.utils.AliCloudImageUtil;
import com.baijiayun.livebase.utils.ThemeDataUtil;
import com.baijiayun.livebase.context.LPConstants;
import com.baijiayun.livecore.models.LPDocExtraModel;
import com.baijiayun.livecore.models.LPMotionEvent;
import com.baijiayun.livecore.models.LPShapeModel;
import com.baijiayun.livecore.models.LPWhiteboardViewInfo;
import com.baijiayun.livecore.models.roomresponse.LPMockClearCacheModel;
import com.baijiayun.livecore.models.roomresponse.LPResRoomDocUpdateModel;
import com.baijiayun.livecore.models.roomresponse.LPResRoomShapeDelModel;
import com.baijiayun.livecore.models.roomresponse.LPResRoomShapeMultipleModel;
import com.baijiayun.livecore.models.roomresponse.LPResRoomShapeSingleModel;
import com.baijiayun.livebase.network.alilog.AliYunLogHelper;
import com.baijiayun.livecore.ppt.util.LPShapeConverter;
import com.baijiayun.livecore.ppt.whiteboard.OnDoubleTapListener2;
import com.baijiayun.livecore.ppt.whiteboard.Whiteboard;
import com.baijiayun.livecore.ppt.whiteboard.WhiteboardView;
import com.baijiayun.livecore.ppt.whiteboard.shape.BitmapShape;
import com.baijiayun.livecore.ppt.whiteboard.shape.DoodleShape;
import com.baijiayun.livecore.ppt.whiteboard.shape.LaserShape;
import com.baijiayun.livecore.ppt.whiteboard.shape.Shape;
import com.baijiayun.livecore.ppt.whiteboard.shape.TextShape;
import com.baijiayun.livecore.viewmodels.impl.LPShapeViewModel;
import com.baijiayun.livecore.viewmodels.impl.ShapeVM;
import com.baijiayun.liveuibase.utils.BaseUtilsKt;

import org.jetbrains.annotations.NotNull;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import android_serialport_api.XYDataPacket;
import io.reactivex.android.schedulers.AndroidSchedulers;
import io.reactivex.disposables.CompositeDisposable;
import io.reactivex.disposables.Disposable;

public class BlackImageWindow implements ShapePaint.IShapePaintListener{

    private BlackboardScrollView scrollView;

    private IRouter iRouter;

    private WhiteboardView whiteboardView;

    private int whiteBoardHeight;

    private ShapeVM shapeVM;

    private ShapeVM.LPShapeReceiverListener lpShapeReceiverListener;

    private List<Shape> bitmapShapes = new ArrayList<>();

    private ShapePaint shapePaint;
    private Disposable paintAuthDisposable;
    private float ratio = 0f;
    private final LPMotionEvent lpMotionEvent = new LPMotionEvent();
    private final CompositeDisposable disposables = new CompositeDisposable();
    private Disposable disposableOfBdsRepaint;
    private Context context;

    public BlackImageWindow(Context context) {
        if (context instanceof RouterListener) {
            iRouter = ((RouterListener) context).getRouter();
            iRouter.getSubjectByKey(EventKey.BitmapShapeList).onNext(bitmapShapes);
        }
        scrollView = onCreateView(context);
        this.context = context;
    }

    public void setShapePaint(ShapePaint shapePaint){
        this.shapePaint = shapePaint;
        shapePaint.registerShapePaintListener(this);
    }

    private int getBlackboardPages() {
        return iRouter.getLiveRoom().getSmallClassTemplateType() == LPConstants.SmallClassTemplateType.ONEONONE ?
                iRouter.getLiveRoom().getPartnerConfig().live1v1BlackboardPages : iRouter.getLiveRoom().getPartnerConfig().liveBlackboardPages;
    }

    public void setOnTouchEvent(XYDataPacket xyDataPacket) {
        if (whiteboardView != null) {
            float singlePageHeight = 1.0f * whiteBoardHeight / getBlackboardPages();
            if (xyDataPacket.ty >= singlePageHeight) {
                xyDataPacket.ty = singlePageHeight;
            }
            xyDataPacket.ty = xyDataPacket.ty + Math.round(ratio * scrollView.getScrollViewHeight());
            lpMotionEvent.setXyDataPacket(xyDataPacket);
            whiteboardView.onTouchEvent(lpMotionEvent);
        }
    }


    public void gotoPrevPage() {
        int remainPages = getBlackboardPages() - 1;
        int page = (int) Math.floor(ratio * remainPages);
        if (page > 0) {
            page--;
            sendScrollSignal(page * 1.0f / remainPages);
        }
    }

    public void gotoNextPage() {
        int remainPages = getBlackboardPages() - 1;
        int page = (int) Math.floor(ratio * remainPages);
        if (page < remainPages) {
            page++;
            sendScrollSignal(page * 1.0f / remainPages);
        }
    }

    public void bdsRepaintShape(String docId) {
        shapeVM.bdsRepaintShape(docId, getBlackboardPages(),
                iRouter.getLiveRoom().getSmallClassTemplateType() == LPConstants.SmallClassTemplateType.ONEONONE?
                        4f/3f : 2f);
    }

    public void initListener() {
        if (lpShapeReceiverListener != null) return;

        if (iRouter.getLiveRoom().getCurrentUser().getType() != LPConstants.LPUserType.Teacher){
            scrollView.setForbidScroll();
        }

        lpShapeReceiverListener = new ShapeVM.LPShapeReceiverListener() {
            @Override
            public void updateShapeInfo(LPResRoomShapeSingleModel lpResRoomShapeSingleModel) {
                if (lpResRoomShapeSingleModel.userId.equals(iRouter.getLiveRoom().getCurrentUser().getUserId()))
                    iRouter.getSubjectByKey(EventKey.ClearToolbarWindow).onNext(true);
                if ("0".equals(lpResRoomShapeSingleModel.docId)) {
                    Shape shape = LPShapeConverter.getShapeFromModel(lpResRoomShapeSingleModel.shape, whiteboardView.getMeasuredWidth(), whiteboardView.getMeasuredHeight(), 0, 0);
                    if (shape instanceof BitmapShape) {
                        bitmapShapes.add(shape);
                        //attention 服务端数据可能有问题导致height大于黑板高度，override宽高之后反而放大了bitmap大小
                        String url = AliCloudImageUtil.getScaledUrl(lpResRoomShapeSingleModel.shape.url, AliCloudImageUtil.SCALED_LFIT, LPConstants.DEFAULT_BITMAP_WIDTH, LPConstants.DEFAULT_BITMAP_WIDTH);
                        Glide.with(whiteboardView.getContext()).asBitmap()
                                .load(url)
                                .into(new SimpleTarget<Bitmap>() {
                                    @Override
                                    public void onResourceReady(@NotNull Bitmap resource, Transition<? super Bitmap> transition) {
                                        AliYunLogHelper.getInstance().addDebugLog("bitmap shape size=" + resource.getAllocationByteCount() / 1024 + "kb");
                                        if(iRouter != null){
                                            ((BitmapShape) shape).setBitmap(resource);
                                            iRouter.getSubjectByKey(EventKey.BitmapShapeList).onNext(bitmapShapes);
                                            whiteboardView.onShapeAdd(shape);
                                        }
                                    }
                                });
                    } else {
                        whiteboardView.onShapeAdd(shape);
                    }
                }
            }

            @Override
            public void updateShape(LPResRoomShapeMultipleModel lpResRoomShapeMultipleModel) {
                if ("0".equals(lpResRoomShapeMultipleModel.docId)) {
                    List<Shape> allList = new ArrayList<>();
                    for (LPShapeModel model : lpResRoomShapeMultipleModel.shapeList) {
                        Shape shape = LPShapeConverter.getShapeFromModel(model, whiteboardView.getMeasuredWidth(), whiteboardView.getMeasuredHeight(), 0, 0);
                        if (shape != null) {
                            allList.add(shape);
                        }
                    }
                    whiteboardView.onShapesUpdate(allList);
                }
            }

            @Override
            public void deleteAllShape(LPResRoomShapeDelModel lpResRoomShapeDelModel) {
                if ("0".equals(lpResRoomShapeDelModel.docId)) {
                    whiteboardView.onShapeClear();
                    bitmapShapes.clear();
                }
            }

            @Override
            public void deleteAllShape(LPMockClearCacheModel lpMockClearCacheModel) {
                // do nothing
            }

            @Override
            public void deleteShape(LPResRoomShapeDelModel lpResRoomShapeDelModel) {
                if ("0".equals(lpResRoomShapeDelModel.docId)) {
                    deleteShape(whiteboardView, lpResRoomShapeDelModel.shapeId);
                }
            }

            private void deleteShape(Whiteboard whiteboard, String shapeIds) {
                whiteboard.onShapeDelete(shapeIds);
                deleteBitmapShape(shapeIds);
            }

            private void deleteBitmapShape(String shapeId) {
                if (bitmapShapes.isEmpty() || TextUtils.isEmpty(shapeId)) return;
                Iterator<Shape> iterator = bitmapShapes.iterator();
                String[] toDeleteShapeArray = shapeId.split(",");
                for(String toDeleteShape : toDeleteShapeArray) {
                    while (iterator.hasNext()) {
                        Shape shape = iterator.next();
                        if (toDeleteShape.equals(shape.id)) {
                            iterator.remove();
                            break;
                        }
                    }
                }
            }

            @Override
            public void drawAllShape(LPResRoomShapeMultipleModel lpResRoomShapeMultipleModel) {
                if ("0".equals(lpResRoomShapeMultipleModel.docId)) {
                    ArrayList<Shape> shapes = new ArrayList<>();
                    if (lpResRoomShapeMultipleModel.shapeList == null) {
                        return;
                    }
                    for (LPShapeModel model : lpResRoomShapeMultipleModel.shapeList) {
                        // 黑板设置高度后，偶现在 7ms 后就 在这里取高度，导致取出高度为 0。
                        Shape shape = LPShapeConverter.getShapeFromModel(model, whiteboardView.getMeasuredWidth(),
                                whiteBoardHeight, 0, 0);
                        if (shape instanceof BitmapShape) {
                            bitmapShapes.add(shape);
                            shapes.add(shape);
                            String url = AliCloudImageUtil.getScaledUrl(model.url, AliCloudImageUtil.SCALED_LFIT, LPConstants.DEFAULT_BITMAP_WIDTH, LPConstants.DEFAULT_BITMAP_WIDTH);
                            Glide.with(whiteboardView.getContext()).asBitmap()
                                    .load(url)
                                    .into(new SimpleTarget<Bitmap>() {
                                        @Override
                                        public void onResourceReady(@NotNull Bitmap resource, Transition<? super Bitmap> transition) {
                                            AliYunLogHelper.getInstance().addDebugLog("bitmap shape size=" + resource.getAllocationByteCount() / 1024 + "kb");
                                            ((BitmapShape) shape).setBitmap(resource);
                                            if (getView() != null) {
                                                getView().post(() -> {
                                                    if (whiteboardView != null && shapes.size() == lpResRoomShapeMultipleModel.shapeList.size()) {
                                                        whiteboardView.onShapeAll(shapes);
                                                    }
                                                });
                                            }
                                        }
                                    });
                        } else if (shape != null) {
                            if(shape instanceof DoodleShape) {
                                ((DoodleShape) shape).isEnd = true;
                            }
                            shapes.add(shape);
                        }
                    }
                    if (bitmapShapes.isEmpty()) {
                        whiteboardView.onShapeAll(shapes);
                    }
                    if(lpResRoomShapeMultipleModel.shapeAppendList != null){
                        for (LPShapeModel model : lpResRoomShapeMultipleModel.shapeAppendList) {
                            Shape shape = LPShapeConverter.getShapeFromModel(model, whiteboardView.getMeasuredWidth(), whiteboardView.getMeasuredHeight(), 0, 0);
                            if (shape instanceof DoodleShape) {
                                whiteboardView.onShapeAppend((DoodleShape) shape, model.smooth);
                            }
                        }
                    }
                }
            }

            @Override
            public void appendShape(LPResRoomShapeSingleModel lpResRoomShapeAppendModel) {
                if("0".equals(lpResRoomShapeAppendModel.docId)){
                    Shape shape = LPShapeConverter.getShapeFromModel(lpResRoomShapeAppendModel.shape, whiteboardView.getMeasuredWidth(),
                            whiteboardView.getMeasuredHeight(), 0, 0);
                    if (shape instanceof DoodleShape) {
                        whiteboardView.onShapeAppend((DoodleShape) shape, lpResRoomShapeAppendModel.shape.smooth);
                    }
                }
            }
        };

        shapeVM = iRouter.getLiveRoom().newShapeVM(lpShapeReceiverListener);
        shapeVM.requestPageAllShape("0", 0);

        if(!BaseUtilsKt.isAdmin(iRouter.getLiveRoom())){
            paintAuthDisposable = iRouter.getLiveRoom().getSpeakQueueVM().getPublishSubjectOfStudentDrawingAuth()
                    .observeOn(AndroidSchedulers.mainThread())
                    .subscribe(aBoolean -> {
                        whiteboardView.setTouchAble(aBoolean);
                        //取消画笔授权，强制发一次shape_add，避免只有shape_append数据
                        if(!aBoolean){
                            whiteboardView.touchEnd();
                        }
                    });
        }
        disposableOfBdsRepaint = ((LPShapeViewModel) shapeVM).getPublishSubjectOfBdsRepaint()
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(aBoolean ->
                        iRouter.getSubjectByKey(EventKey.ReminderMessage).onNext(
                                aBoolean?
                                context.getString(R.string.bjy_group_open_bds_fail) :
                                context.getString(R.string.bjy_group_open_bds_success))
                );
    }

    public void scrollTo(float ratio) {
        this.ratio = ratio;
        scrollView.smoothScrollTo(0, Math.round(ratio * scrollView.getScrollViewHeight()));
    }

    private BlackboardScrollView onCreateView(Context context) {
        BlackboardScrollView scrollView = new BlackboardScrollView(context);
        ScrollView.LayoutParams scrollParams = new ScrollView.LayoutParams(MATCH_PARENT, MATCH_PARENT);
        scrollView.setLayoutParams(scrollParams);
        scrollView.setFillViewport(true);
        scrollView.setOverScrollMode(OVER_SCROLL_NEVER);
        scrollView.setSmoothScrollingEnabled(true);

        whiteboardView = new WhiteboardView(context);
        whiteboardView.onShapeClear();
        whiteboardView.setZoomable(false);
        whiteboardView.setPCBlackboard(true);
        if (iRouter != null) {
            whiteboardView.setUserRole(iRouter.getLiveRoom().getCurrentUser().getType());
            whiteboardView.setEnableStudentOperatePaint(iRouter.getLiveRoom().getPartnerConfig().enableStudentOperatePaint);
        }
        whiteboardView.setAdjustViewBounds(true);
        whiteboardView.setShowPaintOwnerEnable(true);
        whiteboardView.setDrawingRectShapePaintColor(ThemeDataUtil.getColorFromThemeConfigByAttrId(
                context, R.attr.base_theme_window_main_text_color));
        whiteboardView.setOnBitmapShapeListener(bitmapShape -> iRouter.getSubjectByKey(EventKey.BitmapShapeSelected).onNext(bitmapShape));
        whiteboardView.setOnDoubleTapListener(new OnDoubleTapListener2() {
            @Override
            public void onDoubleTapConfirmed() {
                // do nothing
            }

            @Override
            public void onDoubleTapOnShape(Shape shape) {
                iRouter.getSubjectByKey(EventKey.EditTextShape).onNext(((TextShape)shape).getText());
            }
        });

        whiteboardView.setOnViewTapListener((view, x, y) -> {
            if (iRouter != null) {
                iRouter.getSubjectByKey(EventKey.BgSingleTapConfirmed).onNext(true);
            }
        });

        whiteboardView.setShapeSendListener(new WhiteboardView.LPShapeSendListener() {
            @Override
            public void sendShape(Shape shape, LPWhiteboardViewInfo whiteboardViewInfo) {
                LPResRoomShapeSingleModel model = new LPResRoomShapeSingleModel();
                model.shape = LPShapeConverter.getModelFromShape(shape, whiteboardView.getMeasuredWidth(), whiteboardView.getMeasuredHeight(),
                        whiteboardViewInfo.offsetWidth, whiteboardViewInfo.offsetHeight, shape instanceof DoodleShape ? 0 : 1);
                model.docId = "0";
                model.page = 0;
                model.shape.dashType = whiteboardViewInfo.dashType;
                shapeVM.requestShapeAdd(model, whiteboardViewInfo.hasAppend);
            }

            @Override
            public boolean eraseShape(Shape shape, String docId, int page) {
                shapeVM.eraseShape(new LPResRoomShapeDelModel("0", page, shape.id));
                return true;
            }

            @Override
            public void shapeUpdate(List<Shape> shapes, LPWhiteboardViewInfo whiteboardViewInfo) {
                if (shapes == null || shapes.size() == 0) return;
                LPResRoomShapeMultipleModel multipleModel = new LPResRoomShapeMultipleModel();
                List<LPShapeModel> shapeModels = new ArrayList<>();
                for (Shape shape : shapes) {
                    shapeModels.add(LPShapeConverter.getModelFromShape(shape, whiteboardView.getMeasuredWidth(), whiteboardView.getMeasuredHeight(),
                            whiteboardViewInfo.offsetWidth, whiteboardViewInfo.offsetHeight, 1));
                }
                multipleModel.shapeList = shapeModels;
                multipleModel.docId = "0";
                multipleModel.page = 0;
                shapeVM.requestShapeUpdate(multipleModel);
            }

            @Override
            public void laserShapeUpdate(Shape shape, LPWhiteboardViewInfo whiteboardViewInfo) {
                if (shape == null) {
                    return;
                }
                if (shape.isHoverTrace && !iRouter.getLiveRoom().getPartnerConfig().livePainterShowTraceBeforeDraw) {
                    return;
                }
                LPResRoomShapeSingleModel singleModel = new LPResRoomShapeSingleModel();
                singleModel.shape = LPShapeConverter.getModelFromShape(shape, whiteboardView.getMeasuredWidth(), whiteboardView.getMeasuredHeight() / getBlackboardPages(),
                        whiteboardViewInfo.offsetWidth, whiteboardViewInfo.offsetHeight + Math.round(ratio * scrollView.getScrollViewHeight()), 1);
                if (shape.isHoverTrace && iRouter.getLiveRoom().getPartnerConfig().livePainterShowTraceBeforeDraw) {
                    singleModel.shape.setShapeType(LPConstants.ShapeType.Doodle);
                    singleModel.shape.strokeStyle = LPShapeConverter.getColorHex(shape.getPaint().getColor());
                }
                singleModel.docId = "0";
                singleModel.page = 0;
                if (shape instanceof LaserShape) {
                    shapeVM.requestShapeLaserUpdate(singleModel);
                }
            }

            @Override
            public void sendShapeAppend(Shape shape, LPWhiteboardViewInfo whiteboardViewInfo, int index) {
                LPResRoomShapeSingleModel model = new LPResRoomShapeSingleModel();
                model.shape = LPShapeConverter.getModelFromShape(shape, whiteboardView.getMeasuredWidth(), whiteboardView.getMeasuredHeight(),
                        whiteboardViewInfo.offsetWidth, whiteboardViewInfo.offsetHeight, 0);
                model.docId = "0";
                model.page = 0;
                model.shape.index = index;
                model.shape.isEnd = whiteboardViewInfo.isEnd;
                shapeVM.requestShapeAppend(model);
            }
        });
        whiteboardView.setOnShapeSelectedListener(selected -> iRouter.getSubjectByKey(EventKey.Action2ShapeSelected).onNext(selected));

        LinearLayout linearLayout = new LinearLayout(context);
        scrollView.addView(linearLayout, MATCH_PARENT, MATCH_PARENT);
        linearLayout.addView(whiteboardView, MATCH_PARENT, MATCH_PARENT);

        scrollView.setScrollViewListener(new BlackboardScrollView.ScrollViewListener() {
            @Override
            public void onScrollChanged(ScrollView scrollView, int x, int y, int oldx, int oldy) {
                // do nothing
            }

            @Override
            public void onScrollStopped(float ratio) {
                BlackImageWindow.this.ratio = ratio;
                sendScrollSignal(ratio);
            }
        });

        return scrollView;
    }

    private void sendScrollSignal(float ratio) {
        if (iRouter != null && iRouter.getLiveRoom().getCurrentUser().getType() == LPConstants.LPUserType.Teacher) {
            LPResRoomDocUpdateModel lpResRoomDocUpdateModel = new LPResRoomDocUpdateModel();
            lpResRoomDocUpdateModel.docId = "0";
            LPDocExtraModel lpDocExtraModel = new LPDocExtraModel();
            lpDocExtraModel.page = 0;
            lpDocExtraModel.step = 0;
            lpDocExtraModel.scrollTop = String.valueOf(ratio);
            lpResRoomDocUpdateModel.docUpdateExtraModel = lpDocExtraModel;
            iRouter.getLiveRoom().getDocListVM().requestDocUpdate(lpResRoomDocUpdateModel);
        }
    }

    public View getView() {
        return scrollView;
    }

    public void eraseShapes() {
        String deleteIds = whiteboardView.eraseShapes();
        if (TextUtils.isEmpty(deleteIds)) return;
        shapeVM.eraseShape(new LPResRoomShapeDelModel("0", 0, deleteIds));
    }

    public void setPPTEditMode(LPConstants.PPTEditMode editMode){
        whiteboardView.setPPTEditMode(editMode);
    }

    /**
     * delete 图片画笔
     *
     * @param shapeId
     */
    public void eraseBitmapShape(String shapeId) {
        if (TextUtils.isEmpty(shapeId) || shapeVM == null) return;
        shapeVM.eraseShape(new LPResRoomShapeDelModel("0", 0, shapeId));
    }

    /**
     * 清除白板当前页所有画笔
     */
    public void eraseAll() {
        if (shapeVM == null) {
            return;
        }
        shapeVM.eraseAllShape(new LPResRoomShapeDelModel("0", 0));
    }

    public WhiteboardView getWhiteboardView() {
        return whiteboardView;
    }

    public void initWhiteBoardHeight(int height) {
        this.whiteBoardHeight = height;
        whiteboardView.setLayoutParams(new LinearLayout.LayoutParams(
                MATCH_PARENT, height));
    }

    public void onDestroy() {
        //TODO 怎么cancel glide请求， 这里activity.isDestroyed == true
        scrollView = null;
        iRouter = null;
        if(shapePaint != null){
            shapePaint.unregisterShapePaintListener(this);
        }
        whiteboardView.destroy();
        whiteboardView = null;
        if (shapeVM != null)
            ((LPShapeViewModel) shapeVM).removeShapeReceiver(lpShapeReceiverListener);
        shapeVM = null;
        lpShapeReceiverListener = null;
        if (bitmapShapes != null)
            bitmapShapes.clear();
        bitmapShapes = null;
        RxUtils.dispose(disposables);
        RxUtils.dispose(paintAuthDisposable);
        RxUtils.dispose(disposableOfBdsRepaint);
    }

    @Override
    public void onShapeCustomStrokeWidthChange(float strokeWidth) {
        whiteboardView.setCustomShapeStrokeWidth(strokeWidth);
    }

    @Override
    public void onShapeDoodleStrokeWidthChange(float strokeWidth) {
        whiteboardView.setShapeStrokeWidth(strokeWidth);
    }

    @Override
    public void onShapeColorChange(int shapeColor) {
        whiteboardView.setShapeColor(shapeColor);
    }

    @Override
    public void onShapeTypeChange(LPConstants.ShapeType shapeType) {
        whiteboardView.setCustomShapeType(shapeType);
    }

    @Override
    public void onShapeEditModeChange(LPConstants.PPTEditMode pptEditMode) {
        whiteboardView.setPPTEditMode(pptEditMode);
    }

    @Override
    public void onShapeTextSizeChange(int textSize) {
        whiteboardView.setPaintTextSize(textSize);
    }

    @Override
    public void onSendDrawTextConfirmed(String beforeText, String content) {
        whiteboardView.sendDrawTextConfirmed(beforeText, content);
    }
}
