package com.baijiayun.groupclassui.window.group

import android.content.Context
import android.graphics.Color
import android.view.View
import android.view.ViewGroup
import android.widget.FrameLayout
import android.widget.TextView
import com.baijiahulian.common.cropperv2.uikit.CircleColorView
import com.baijiayun.bjyutils.drawable.DrawableBuilder
import com.baijiayun.bjyutils.kt.dp
import com.baijiayun.bjyutils.screen.DisplayUtils
import com.baijiayun.groupclassui.R
import com.baijiayun.groupclassui.model.GroupAwardInfoModel
import com.baijiayun.groupclassui.window.InteractiveBaseWindow
import com.baijiayun.livebase.utils.ThemeDataUtil
import com.baijiayun.livecore.models.LPGroupItem
import kotlinx.android.synthetic.main.bjy_window_group_award.view.*

/**
 * Created by pengwenjian on 2020/7/23
 * Describe: 分组点赞
 */
class GroupAwardWindow(context: Context) : InteractiveBaseWindow(context) {
    private val LEFT_TOP = 1
    private val RIGHT_TOP = 2
    private val LEFT_BOTTOM = -1
    private val RIGHT_BOTTOM = -2
    private var groupPopupState = LEFT_TOP
    private var parentView: ViewGroup? = null
    private var contentView: View? = null

    override fun onCreateView(context: Context?): View = View.inflate(context, R.layout.bjy_window_group_award, null)

    init {
        setAllowTouch(true)
        setOnSingleTapListener {
            if (isShowInfoWindow()) {
                parentView?.removeView(contentView)
            } else {
                showGroupInfoPopupWindow()
            }
        }
        //移动过程中调整info
        setOnWindowRepositionListener(object : OnWindowRepositionListener {
            override fun OnWindowScale(newWidth: Int, newHeight: Int) {
                // do nothing
            }

            override fun onWindowMove(x: Int, y: Int) {
                if (!isShowInfoWindow()) {
                    return
                }
                val parentView = view.parent
                if (parentView !is ViewGroup) {
                    return
                }
                val contentWidth: Int = context.resources.getDimensionPixelSize(R.dimen.bjysc_award_info_width)
                val contentHeight: Int = context.resources.getDimensionPixelSize(R.dimen.bjysc_award_info_height)
                val windowView: View = view
                val viewX = windowView.left
                val viewY = windowView.top
                val right = windowView.right
                //需要调节的情况只有4种
                //1.高度超出 底部超出
                if (viewY + contentHeight >= parentView.measuredHeight && isStateTop()) {
                    groupPopupState = -groupPopupState
                }
                //2.高度超出 顶部超出
                if (viewY <= contentHeight - windowView.measuredHeight && !isStateTop()) {
                    groupPopupState = -groupPopupState
                }
                //3.宽度超出 右边超出
                if (right + contentWidth >= parentView.measuredWidth && !isStateLeft()) {
                    groupPopupState /= 2
                }
                //4.宽度超出 左边超出
                if (viewX <= contentWidth && isStateLeft()) {
                    groupPopupState *= 2
                }
                val layoutParams = contentView?.layoutParams
                if (layoutParams !is FrameLayout.LayoutParams) {
                    return
                }
                when (groupPopupState) {
                    LEFT_TOP -> {
                        layoutParams.topMargin = view.top
                        layoutParams.leftMargin = view.left - contentWidth
                    }
                    LEFT_BOTTOM -> {
                        layoutParams.topMargin = view.bottom - contentHeight
                        layoutParams.leftMargin = view.left - contentWidth
                    }
                    RIGHT_TOP -> {
                        layoutParams.topMargin = view.top
                        layoutParams.leftMargin = view.right
                    }
                    RIGHT_BOTTOM -> {
                        layoutParams.topMargin = view.bottom - contentHeight
                        layoutParams.leftMargin = view.right
                    }
                }
            }
        })
    }

    override fun onRoomStatusChange(isActive: Boolean): Boolean {
        if (isActive) {
            val parent = view.parent
            if (parent is ViewGroup) {
                parentView = parent
            }
            contentView = View.inflate(context,R.layout.bjy_layout_award_group_info,null)
            contentView?.background = (DrawableBuilder()
                    .cornerRadius(4.dp)
                    .solidColor(ThemeDataUtil.getColorFromThemeConfigByAttrId(context, R.attr.base_theme_window_bg_color))
                    .build())
        }
        return super.onRoomStatusChange(isActive)
    }

    private fun isStateTop() = groupPopupState == LEFT_TOP || groupPopupState == RIGHT_TOP
    private fun isStateLeft() = groupPopupState == LEFT_TOP || groupPopupState == LEFT_BOTTOM

    fun setGroupAwardCount(groupAwardCount: Int) {
        view.tvAwardCount.text = groupAwardCount.toString()
    }

    private fun showGroupInfoPopupWindow() {
        val contentWidth: Int = context.resources.getDimensionPixelSize(R.dimen.bjysc_award_info_width)
        val contentHeight: Int = context.resources.getDimensionPixelSize(R.dimen.bjysc_award_info_height)
        val right = view.right
        val top = view.top
        var adjustWidth = false
        var adjustHeight = false
        if (parentView !is ViewGroup) {
            return
        }
        //原本的方向是RIGHT_TOP
        //高度超出
        if (top + contentHeight >= parentView!!.measuredHeight) {
            adjustHeight = true
        }
        //宽度超出
        if (right + contentWidth >= parentView!!.measuredWidth) {
            adjustWidth = true
        }
        val layoutParams = FrameLayout.LayoutParams(contentWidth,contentHeight)
        groupPopupState = if (adjustHeight && adjustWidth) {
            layoutParams.topMargin = view.bottom - contentHeight
            layoutParams.leftMargin = view.left - contentWidth
            LEFT_BOTTOM
        } else if (adjustHeight) {
            layoutParams.topMargin = view.bottom - contentHeight
            layoutParams.leftMargin = view.right
            RIGHT_BOTTOM
        } else if (adjustWidth) {
            layoutParams.topMargin = view.top
            layoutParams.leftMargin = view.left - contentWidth
            LEFT_TOP
        } else {
            layoutParams.topMargin = view.top
            layoutParams.leftMargin = view.right
            RIGHT_TOP
        }
        parentView!!.addView(contentView, layoutParams)
        setGroupAwardContent(getGroupAwardInfoModel())
    }

    /**
     * 获取自己组排名，可重复，如 1  2  2  2  5
     *
     * @param groupId
     * @return
     */
    private fun getGroupRank(groupId: Int): Int {
        var rank = 0
        var count = -1
        var same = 0
        val awardGroupMap: Map<Int, Int> = iRouter.liveRoom.toolBoxVM.awardGroupMap
        val groupItems: MutableList<LPGroupItem> = ArrayList()
        for (key in awardGroupMap.keys) {
            val value = awardGroupMap[key]
            val groupItem = LPGroupItem(key)
            groupItem.count = value!!
            groupItems.add(groupItem)
        }
        groupItems.sortWith { o1: LPGroupItem, o2: LPGroupItem -> o2.count - o1.count }
        for (groupItem in groupItems) {
            if (groupItem.count == count) {
                same++
            } else {
                rank += same + 1
                count = groupItem.count
                same = 0
            }
            if (groupItem.id == groupId) {
                break
            }
        }
        return rank
    }

    private fun setGroupAwardContent(groupAwardInfoModel: GroupAwardInfoModel?) {
        contentView?.let {
            val tvGroupName = it.findViewById<TextView>(R.id.tv_group_name)
            val circleColorView: CircleColorView = it.findViewById(R.id.view_group_color)
            val tvGroupAwardCount = it.findViewById<TextView>(R.id.tv_group_award_count)
            val tvGroupCount = it.findViewById<TextView>(R.id.tv_group_count)
            val tvGroupRank = it.findViewById<TextView>(R.id.tv_group_rank)
            tvGroupName.text = groupAwardInfoModel?.groupName
            groupAwardInfoModel?.run { circleColorView.setColor(groupColor) }
            tvGroupAwardCount.text = groupAwardInfoModel?.groupAwardCount.toString()
            tvGroupCount.text = context.getString(R.string.bjysc_group_count, groupAwardInfoModel?.groupCount
                    ?: 0)
            tvGroupRank.text = context.getString(R.string.bjysc_group_rank, groupAwardInfoModel?.groupRank
                    ?: 1)
        }
    }

    private fun getGroupAwardInfoModel(): GroupAwardInfoModel? {
        val groupAwardInfoModel = GroupAwardInfoModel()
        val groupId: Int = iRouter.liveRoom.currentUser.group
        val lpGroupItem: LPGroupItem = iRouter.liveRoom.onlineUserVM.groupMap.get(groupId)
        val groupAwardCount: Int = iRouter.liveRoom.toolBoxVM.awardGroupMap[groupId] ?: 0
        groupAwardInfoModel.groupId = groupId
        groupAwardInfoModel.groupName = lpGroupItem.groupItemModel.name
        groupAwardInfoModel.groupCount = lpGroupItem.count
        groupAwardInfoModel.groupAwardCount = groupAwardCount
        try {
            groupAwardInfoModel.groupColor = Color.parseColor(lpGroupItem.groupItemModel.color)
        } catch (e: Exception) {
            e.printStackTrace()
        }
        groupAwardInfoModel.groupRank = getGroupRank(groupId)
        return groupAwardInfoModel
    }

    fun updateGroupAwardContent() {
        if (isShowInfoWindow()) {
            getGroupAwardInfoModel()?.let { setGroupAwardContent(it) }
        }
    }

    private fun isShowInfoWindow() = parentView?.indexOfChild(contentView) != -1

    fun dismissInfoWindow() {
        if (isShowInfoWindow()) {
            parentView?.removeView(contentView)
        }
    }

    override fun onDestroy() {
        super.onDestroy()
        dismissInfoWindow()
    }
}