package com.baijiayun.groupclassui.window.group;

import static com.baijiayun.bjyutils.kt.UtilsKt.getEncodePhoneNumber;

import android.content.Context;
import android.graphics.drawable.Drawable;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.LinearLayout;
import android.widget.TextView;

import androidx.annotation.IntRange;
import androidx.annotation.NonNull;
import androidx.appcompat.widget.AppCompatImageView;
import androidx.core.content.ContextCompat;
import androidx.viewpager.widget.PagerAdapter;
import androidx.viewpager.widget.ViewPager;

import com.baijiayun.bjyutils.drawable.DrawableBuilder;
import com.baijiayun.bjyutils.drawable.StateListDrawableBuilder;
import com.baijiayun.glide.Glide;
import com.baijiayun.glide.request.RequestOptions;
import com.baijiayun.groupclassui.R;
import com.baijiayun.livebase.utils.ThemeDataUtil;
import com.baijiayun.livebase.models.imodels.IUserModel;

import java.util.ArrayList;
import java.util.List;

/**
 * 已分组PagerAdapter，大于5个就新增1列
 * Created by Dujuncan on 2018/09/30.
 */
public class GroupManagePagerAdapter extends PagerAdapter {
    private Context context;
    private ViewPager mViewPager;
    private List<IUserModel> groupUsers;
    private List<IUserModel> selectedUsers;//选中的组内用户
    private OnSelectedListener onSelectedListener;

    public GroupManagePagerAdapter(Context context, List<IUserModel> groupUsers) {
        this.context = context;
        this.groupUsers = groupUsers;
        selectedUsers = new ArrayList<>();
    }

    @Override
    public int getCount() {
        if (groupUsers == null) return 0;
        //1-5   1
        //6-10  2
        //11-15 3
        int count = 0;
        if (groupUsers.size() > 10) {
            count = 3;
        } else if (groupUsers.size() > 5) {
            count = 2;
        } else if (groupUsers.size() > 0) {
            count = 1;
        }
        return count;
    }

    @Override
    public int getItemPosition(@NonNull Object object) {
        selectedUsers.clear();
        return POSITION_NONE;
    }

    @NonNull
    @Override
    public Object instantiateItem(@NonNull ViewGroup container, int position) {
        View item = LayoutInflater.from(context).inflate(R.layout.item_group_manage_window_viewpager, container, false);
        LinearLayout ll = item.findViewById(R.id.item_group_manage_window_viewpager_ll);

        //根据对应viewpager的index取出的数据，添加到linearLayout中
        List<IUserModel> rowUser = getUsers(position);
        for (int i = 0; i < rowUser.size(); i++) {
            Drawable selectedDrawable = ThemeDataUtil.getCommonWindowBg(context);
            Drawable normalDrawable = new DrawableBuilder()
                    .solidColor(ContextCompat.getColor(context,R.color.bjysc_transparent))
                    .build();
            View usersView = LayoutInflater.from(context).inflate(R.layout.item_group_window_user, null);
            usersView.setBackground(new StateListDrawableBuilder()
                    .normal(normalDrawable)
                    .selected(selectedDrawable)
                    .build());
            TextView tvName = usersView.findViewById(R.id.item_group_window_tv_name);
            AppCompatImageView ivHead = usersView.findViewById(R.id.item_group_window_iv_head);
            tvName.setText(getEncodePhoneNumber(rowUser.get(i).getName()));
            Glide.with(context.getApplicationContext()).load(rowUser.get(i).getAvatar())
                    .apply(RequestOptions.circleCropTransform())
                    .into(ivHead);
            int currIndex = i;
            usersView.setOnClickListener(v -> {
                usersView.setSelected(!usersView.isSelected());
                if (usersView.isSelected()) {
                    selectedUsers.add(rowUser.get(currIndex));
                } else if (selectedUsers.size() > 0) {
                    selectedUsers.remove(rowUser.get(currIndex));
                }
                if (onSelectedListener != null) {
                    onSelectedListener.onSelected(selectedUsers);
                }
            });
            ll.addView(usersView);
        }
        container.addView(item);
        return item;
    }

    @Override
    public void destroyItem(@NonNull ViewGroup container, int position, @NonNull Object object) {
        container.removeView((View) object);
    }

    @Override
    public boolean isViewFromObject(@NonNull View view, @NonNull Object object) {
        return view == object;
    }

    public void bindToViewPager(ViewPager viewPager) {
        mViewPager = viewPager;
        viewPager.setAdapter(this);
    }

    public void addData(IUserModel groupUser) {
        if (groupUsers == null || groupUser == null) return;
        groupUsers.add(groupUser);
        notifyDataSetChanged();
    }

    public void addData(List<IUserModel> users) {
        if (groupUsers == null || users == null) return;
        groupUsers.addAll(users);
        notifyDataSetChanged();
    }

    public void removeData(List<IUserModel> users) {
        if (groupUsers == null || users == null) return;
        groupUsers.removeAll(users);
        notifyDataSetChanged();
    }

    public int getCurrentItem() {
        if (mViewPager == null) return -1;
        return mViewPager.getCurrentItem();
    }

    public List<IUserModel> getGroupUsers() {
        return groupUsers;
    }

    //根据index，找出对应页的5个数据
    private List<IUserModel> getUsers(@IntRange(from = 0, to = 2) int position) {
        List<IUserModel> tempUsers = new ArrayList<>();
        switch (position) {
            case 0:
                if (groupUsers.size() <= 5) tempUsers = groupUsers;
                if (groupUsers.size() > 5) {
                    tempUsers = groupUsers.subList(0, 5);
                }
                break;
            case 1:
                if (groupUsers.size() <= 10) tempUsers = groupUsers.subList(5, groupUsers.size());
                if (groupUsers.size() > 10) tempUsers = groupUsers.subList(5, 10);
                break;
            case 2:
                tempUsers = groupUsers.subList(10, groupUsers.size());
                break;
            default:
                break;
        }
        return tempUsers;
    }

    public interface OnSelectedListener {
        void onSelected(List<IUserModel> selectUsers);
    }

    public void setOnSelectedListener(OnSelectedListener onSelectedListener) {
        this.onSelectedListener = onSelectedListener;
    }
}
