package com.baijiayun.groupclassui.window.group;

import android.content.Context;
import android.view.LayoutInflater;
import android.view.View;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;

import androidx.appcompat.widget.AppCompatSpinner;
import androidx.core.content.ContextCompat;

import com.baijiayun.bjyutils.drawable.DrawableBuilder;
import com.baijiayun.bjyutils.kt.UtilsKt;
import com.baijiayun.groupclassui.R;
import com.baijiayun.groupclassui.global.EventKey;
import com.baijiayun.groupclassui.window.InteractiveBaseWindow;
import com.baijiayun.livebase.utils.ThemeDataUtil;
import com.baijiayun.livecore.models.imodels.IMediaModel;

import java.util.ArrayList;
import java.util.List;

/**
 * 设置分组数量window
 * Created by Dujuncan on 2018/09/28.
 */
public class SetGroupCountWindow extends InteractiveBaseWindow {

    private OnNextStepListener onNextStepListener;
    private int selectedGroupCount;//选择的分组数

    public SetGroupCountWindow(Context context) {
        super(context);

        $.id(R.id.window_set_group_tv_cancel).clicked(v -> {
            iRouter.getSubjectByKey(EventKey.GroupManageLayout)
                    .onNext(false);
        });
        $.id(R.id.window_set_group_tv_next_step).clicked(v -> {
            if (onNextStepListener != null) {
                onNextStepListener.onNextStep(selectedGroupCount);
            }
        }).view().setBackground(new DrawableBuilder()
                .solidColor(ThemeDataUtil.getColorFromThemeConfigByAttrId(context, R.attr.base_theme_live_product_color))
                .cornerRadius(UtilsKt.getDp(4))
                .build());
        $.id(R.id.window_set_group_root_container).view().setBackground(new DrawableBuilder()
                .strokeWidth(UtilsKt.getDp(1))
                .strokeColor(ThemeDataUtil.getColorFromThemeConfigByAttrId(context, R.attr.base_theme_window_assistant_text_color))
                .cornerRadius(UtilsKt.getDp(4))
                .solidColor(ContextCompat.getColor(context,R.color.bjysc_half_black))
                .build());
        //禁用移动
        view.setOnTouchListener(null);

        setSpinner(context, iRouter.getLiveRoom().getSpeakQueueVM().getSpeakQueueList());
    }

    private void setSpinner(Context context, List<IMediaModel> speakQueueLis) {
        List<Integer> counts = new ArrayList<>();
        //为了能选择0组，首先加入0
        counts.add(0);
        for (int i = 1; i <= speakQueueLis.size(); i++) {
            counts.add(i);
        }

        AppCompatSpinner spinner = $.id(R.id.window_set_group_spinner).view();
        ArrayAdapter adapter = new ArrayAdapter<>(context, R.layout.set_group_spinner_item, R.id.window_set_group_spinner_tv, counts);
        adapter.setDropDownViewResource(R.layout.set_group_spinner_dropdown_item);
        spinner.setAdapter(adapter);
        spinner.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {
            @Override
            public void onItemSelected(AdapterView<?> parent, View view, int position, long id) {
                selectedGroupCount = counts.get(parent.getSelectedItemPosition());
            }

            @Override
            public void onNothingSelected(AdapterView<?> parent) {
                // do nothing
            }
        });
        //因为开始加入了第0组，所以用户数需要-1.
        $.id(R.id.window_set_group_tv_curr_count)
                .text(context.getString(R.string.bjysc_group_set_count_tips, counts.size() - 1, counts.size() - 1));
    }

    @Override
    protected View onCreateView(Context context) {
        return LayoutInflater.from(context).inflate(R.layout.window_set_group_count, null);
    }

    public interface OnNextStepListener {
        void onNextStep(int groupCount);
    }

    public void setOnNextStepListener(OnNextStepListener onNextStepListener) {
        this.onNextStepListener = onNextStepListener;
    }
}
