package com.baijiayun.groupclassui.window.group;

import static com.baijiayun.bjyutils.kt.UtilsKt.getEncodePhoneNumber;

import android.annotation.SuppressLint;
import android.content.Context;
import android.graphics.drawable.Drawable;
import android.util.Log;
import android.util.SparseBooleanArray;
import android.view.GestureDetector;
import android.view.LayoutInflater;
import android.view.MotionEvent;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;

import androidx.annotation.NonNull;
import androidx.core.content.ContextCompat;
import androidx.core.view.GestureDetectorCompat;

import com.baijiayun.bjyutils.drawable.DrawableBuilder;
import com.baijiayun.bjyutils.drawable.StateListDrawableBuilder;
import com.baijiayun.glide.Glide;
import com.baijiayun.glide.request.RequestOptions;
import com.baijiayun.groupclassui.R;
import com.baijiayun.groupclassui.base.BaseRecyclerAdapter;
import com.baijiayun.groupclassui.base.BaseViewHolder;
import com.baijiayun.livebase.utils.ThemeDataUtil;
import com.baijiayun.livebase.models.imodels.IUserModel;

import java.util.ArrayList;
import java.util.List;

/**
 * 未分组用户adapter
 * Created by Dujuncan on 2018/9/27.
 */
public class UngroupUserAdapter extends BaseRecyclerAdapter<IUserModel> {

    private static final String TAG = BaseRecyclerAdapter.class.getCanonicalName();
    private OnUserEventListener mOnUserEventListener;
    private List<IUserModel> mSelectedUser;//选中的未分组用户
    private SparseBooleanArray selectedStateArray;

    public UngroupUserAdapter(Context context) {
        super(context, R.layout.item_group_window_user);
        mSelectedUser = new ArrayList<>();
        selectedStateArray = new SparseBooleanArray();

    }

    @NonNull
    @Override
    public BaseViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        View view = LayoutInflater.from(mContext).inflate(R.layout.item_group_window_user, parent, false);
        Drawable selectedDrawable = new DrawableBuilder()
                .solidColor(ThemeDataUtil.getColorFromThemeConfigByAttrId(mContext, R.attr.base_theme_window_bg_color))
                .cornerRadius(mContext.getResources().getDimensionPixelSize(R.dimen.bjy_base_common_bg_radius))
                .build();
        Drawable normalDrawable = new DrawableBuilder()
                .solidColor(ContextCompat.getColor(mContext,R.color.bjysc_transparent))
                .build();
        view.findViewById(R.id.item_group_user_root_container).setBackground(new StateListDrawableBuilder()
                .normal(normalDrawable)
                .selected(selectedDrawable)
                .build());
        return new BaseViewHolder(mContext, view);
    }

    @SuppressLint("ClickableViewAccessibility")
    @Override
    protected void convert(BaseViewHolder holder, IUserModel groupUser) {
        View view = holder.getRootView();
        view.setSelected(selectedStateArray.get(holder.getAdapterPosition()));
        UserListGestureListener gestureListener = new UserListGestureListener(view, holder.getAdapterPosition());
        GestureDetectorCompat gestureDetectorCompat = new GestureDetectorCompat(mContext.getApplicationContext(), gestureListener);
        view.setOnTouchListener((v, event) -> {
            gestureDetectorCompat.onTouchEvent(event);
            return true;
        });
        holder.setText(R.id.item_group_window_tv_name, getEncodePhoneNumber(groupUser.getName()));
        Glide.with(mContext.getApplicationContext()).load(groupUser.getAvatar())
                .apply(RequestOptions.circleCropTransform())
                .into((ImageView) holder.getView(R.id.item_group_window_iv_head));
    }

    @Override
    protected boolean isEnableListener() {
        return false;
    }

    //用户长按监听
    public interface OnUserEventListener {
        void onUserSelected(List<IUserModel> selectedUsers);

        void onUserLongClick(View view, int position, float viewLeft, float viewTop, float rawX, float rawY);
    }

    public void setOnUserEventListener(OnUserEventListener onUserLongClickListener) {
        mOnUserEventListener = onUserLongClickListener;
    }

    class UserListGestureListener extends GestureDetector.SimpleOnGestureListener {

        private int position;
        private View view;

        public UserListGestureListener(View view, int position) {
            this.position = position;
            this.view = view;
            Log.d(TAG, "UserListGestureListener: " + position);
        }

        @Override
        public boolean onSingleTapUp(MotionEvent e) {
            //每次单击切换选中状态
            view.setSelected(!view.isSelected());
            if (view.isSelected()) {
                mSelectedUser.add(getData().get(position));
                selectedStateArray.put(position, true);
            } else {
                mSelectedUser.remove(getData().get(position));
                selectedStateArray.put(position, false);
            }
            if (mOnUserEventListener != null) {
                mOnUserEventListener.onUserSelected(mSelectedUser);
            }
            return super.onSingleTapUp(e);
        }

        @Override
        public void onLongPress(MotionEvent e) {
            super.onLongPress(e);
            if (mOnUserEventListener != null) {
                mOnUserEventListener.onUserLongClick(view, position, e.getRawX() - e.getX(), e.getRawY() - e.getY(), e.getRawX(), e.getRawY());
            }
        }
    }

    //清空选中数据，清除选中状态
    public void clearSelectState() {
        mSelectedUser.clear();
        selectedStateArray.clear();
    }
}
