package com.baijiayun.groupclassui.window.multiupload;

import android.content.Context;
import android.content.pm.ActivityInfo;
import android.text.InputFilter;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.constraintlayout.widget.ConstraintLayout;
import androidx.constraintlayout.widget.ConstraintSet;
import androidx.recyclerview.widget.GridLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.baijiahulian.common.cropperv2.BJCommonImageCropHelper;
import com.baijiahulian.common.cropperv2.ThemeConfig;
import com.baijiahulian.common.cropperv2.model.PhotoInfo;
import com.baijiayun.bjyutils.kt.UtilsKt;
import com.baijiayun.bjyutils.screen.DisplayUtils;
import com.baijiayun.glide.Glide;
import com.baijiayun.groupclassui.R;
import com.baijiayun.groupclassui.global.EventKey;
import com.baijiayun.groupclassui.window.InteractiveBaseWindow;
import com.baijiayun.livebase.utils.ThemeDataUtil;
import com.baijiayun.liveuibase.chat.LengthFilter;

import java.util.ArrayList;
import java.util.List;

/**
 * Created by pengwenjian on 2021/3/8
 * Describe:
 * 文字图片上传窗口
 */
public class MultiUploadWindow extends InteractiveBaseWindow implements IMultiUploadListener {
    private static final int LENGTH_LIMIT = 300;
    private ImgAdapter imgAdapter;
    private boolean isUploaded = false;
    private int imgCountLimit = 5;

    public MultiUploadWindow(Context context) {
        super(context);
        initView();
    }

    private void initView() {
        RecyclerView recyclerView = $.id(R.id.recycler_view).view();
        imgAdapter = new ImgAdapter();
        recyclerView.setLayoutManager(new GridLayoutManager(context, imgCountLimit));
        recyclerView.setAdapter(imgAdapter);
        recyclerView.addItemDecoration(new GridSpacingItemDecoration(imgCountLimit, UtilsKt.getDp(6), false));
        EditText editText = $.id(R.id.et_question).view();
        LengthFilter lengthFilter = new LengthFilter(context, LENGTH_LIMIT);
        lengthFilter.setTip(context.getString(R.string.bjysc_enter_limit_less, LENGTH_LIMIT));
        editText.setFilters(new InputFilter[]{lengthFilter});
    }

    public void showTitle(boolean showTitle) {
        $.id(R.id.title_container).visibility(showTitle ? View.VISIBLE : View.GONE);
    }

    public void showTitleIcon(boolean show) {
        $.id(R.id.tv_title_new).visibility(show ? View.VISIBLE : View.GONE);
    }

    public void setTitle(String title) {
        $.id(R.id.tv_title).text(title);
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
    }

    @Override
    protected View onCreateView(Context context) {
        return View.inflate(context, R.layout.bjysc_window_upload_multi, null);
    }

    @Override
    public String getContent() {
        return ((TextView) $.id(R.id.et_question).view()).getText().toString();
    }

    @Override
    public void setContent(String content) {
        $.id(R.id.et_question).text(content);
        $.id(R.id.tv_question).text(content);
    }

    @Override
    public List<String> getLocalImgPaths() {
        return imgAdapter.getLocalImgPaths();
    }

    @Override
    public List<String> getRemoteImgPaths() {
        return imgAdapter.getRemoteImgPaths();
    }

    @Override
    public void setRemoteImgPaths(List<String> imgPaths) {
        if (imgPaths == null || imgPaths.isEmpty()) {
            if (getLocalImgPaths() == null || getLocalImgPaths().isEmpty()) {
                $.id(R.id.recycler_view).gone();
            }
            imgAdapter.setRemoteData(new ArrayList<>());
            return;
        }
        imgAdapter.setRemoteData(imgPaths);
    }

    @Override
    public void refresh() {
        $.id(R.id.recycler_view).visible();
        imgAdapter.notifyDataSetChanged();
    }

    @Override
    public void setUploaded(boolean isUploaded) {
        setUploaded(isUploaded, false);
    }

    @Override
    public void setUploaded(boolean isUploaded, boolean isFillParent) {
        this.isUploaded = isUploaded;
        if (isUploaded) {
            if (isFillParent) {
                $.id(R.id.et_question).visible().enable(false);
                $.id(R.id.tv_question).gone();
                ConstraintLayout.LayoutParams layoutParams = (ConstraintLayout.LayoutParams) $.id(R.id.recycler_view).view().getLayoutParams();
                layoutParams.topToBottom = ConstraintSet.UNSET;
                layoutParams.bottomToBottom = ConstraintSet.PARENT_ID;
            }else {
                $.id(R.id.et_question).gone();
                $.id(R.id.tv_question).visible();
                ConstraintLayout.LayoutParams layoutParams = (ConstraintLayout.LayoutParams) $.id(R.id.recycler_view).view().getLayoutParams();
                layoutParams.bottomToBottom = ConstraintSet.UNSET;
                layoutParams.topToBottom = R.id.tv_question;
            }
        } else {
            $.id(R.id.et_question).visible().enable(true);
            $.id(R.id.tv_question).gone();
            ConstraintLayout.LayoutParams layoutParams = (ConstraintLayout.LayoutParams) $.id(R.id.recycler_view).view().getLayoutParams();
            layoutParams.topToBottom = ConstraintSet.UNSET;
            layoutParams.bottomToBottom = ConstraintSet.PARENT_ID;
        }
    }

    @Override
    public void setImgCountLimit(int imgCountLimit) {
        this.imgCountLimit = imgCountLimit;
    }

    class ImgAdapter extends RecyclerView.Adapter<RecyclerView.ViewHolder> {
        private final List<String> imgPaths = new ArrayList<>();
        private final List<String> localImgPaths = new ArrayList<>();
        private List<String> remoteImgPaths = new ArrayList<>();
        private static final int VIEW_TYPE_IMG = 1;
        private static final int VIEW_TYPE_ADD = 2;

        public void setRemoteData(List<String> remoteImgPaths) {
            this.imgPaths.clear();
            this.localImgPaths.clear();
            this.remoteImgPaths = remoteImgPaths;
            this.imgPaths.addAll(remoteImgPaths);
            notifyDataSetChanged();
        }

        public List<String> getLocalImgPaths() {
            return localImgPaths;
        }

        public List<String> getRemoteImgPaths() {
            return remoteImgPaths;
        }

        @NonNull
        @Override
        public RecyclerView.ViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
            if (viewType == VIEW_TYPE_IMG) {
                View view = LayoutInflater.from(parent.getContext()).inflate(R.layout.bjysc_item_img_upload, parent, false);
                DisplayUtils.setRoundCorner(view, context.getResources().getDimensionPixelSize(R.dimen.bjy_base_common_bg_radius));
                return new ImgViewHolder(view);
            } else {
                View view = LayoutInflater.from(parent.getContext()).inflate(R.layout.bjysc_item_img_add, parent, false);
                DisplayUtils.setRoundCorner(view, context.getResources().getDimensionPixelSize(R.dimen.bjy_base_common_bg_radius));
                return new AddViewHolder(view);
            }
        }

        @Override
        public void onBindViewHolder(@NonNull RecyclerView.ViewHolder holder, int position) {
            if (holder instanceof ImgViewHolder) {
                ImgViewHolder imgViewHolder = (ImgViewHolder) holder;
                String url = imgPaths.get(position);
                Glide.with(context).load(url).into(imgViewHolder.ivSrc);
                imgViewHolder.ivDelete.setVisibility(isUploaded ? View.INVISIBLE : View.VISIBLE);
                imgViewHolder.ivDelete.setOnClickListener(v -> {
                    imgPaths.remove(url);
                    localImgPaths.remove(url);
                    remoteImgPaths.remove(url);
                    notifyDataSetChanged();
                });
                imgViewHolder.ivSrc.setOnClickListener(v -> iRouter.getSubjectByKey(EventKey.DisplayImgPreview).onNext(url));
            } else {
                AddViewHolder addViewHolder = (AddViewHolder) holder;
                addViewHolder.ivAdd.setOnClickListener(v -> {
                    ThemeConfig.Builder builder = new ThemeConfig.Builder();
                    builder.setScreenOrientation(ActivityInfo.SCREEN_ORIENTATION_LANDSCAPE)
                            .setMainElementsColor(ThemeDataUtil.getColorFromThemeConfigByAttrId(context, R.attr.base_theme_live_product_color));
                    BJCommonImageCropHelper.openImageMulti(context, imgCountLimit - imgPaths.size(), builder.build(), new BJCommonImageCropHelper.OnHandlerResultCallback() {

                        @Override
                        public void onHandlerSuccess(List<PhotoInfo> list) {
                            for (PhotoInfo photoInfo : list) {
                                localImgPaths.add(photoInfo.getPhotoPath());
                                imgPaths.add(photoInfo.getPhotoPath());
                            }
                            notifyDataSetChanged();
                        }

                        @Override
                        public void onHandlerFailure(String s) {
                            showToastMessage(s);
                        }
                    });
                });
            }
        }

        @Override
        public int getItemCount() {
            int count;
            if (isUploaded) {
                count = imgPaths.size();
            } else {
                if (imgPaths.isEmpty()) {
                    count = 1;
                } else {
                    if (imgPaths.size() < imgCountLimit) {
                        count = imgPaths.size() + 1;
                    } else {
                        count = imgCountLimit;
                    }
                }
            }
            return count;
        }

        @Override
        public int getItemViewType(int position) {
            if (position < imgPaths.size()) {
                return VIEW_TYPE_IMG;
            } else {
                return VIEW_TYPE_ADD;
            }
        }
    }

    static class ImgViewHolder extends RecyclerView.ViewHolder {
        ImageView ivDelete;
        ImageView ivSrc;

        public ImgViewHolder(@NonNull View itemView) {
            super(itemView);
            ivDelete = itemView.findViewById(R.id.iv_delete);
            ivSrc = itemView.findViewById(R.id.iv);
        }
    }

    static class AddViewHolder extends RecyclerView.ViewHolder {
        ImageView ivAdd;

        public AddViewHolder(@NonNull View itemView) {
            super(itemView);
            ivAdd = itemView.findViewById(R.id.iv_add);
        }
    }
}
