package com.baijiayun.groupclassui.window.status;

import android.content.Context;

import com.baijiayun.bjyrtcengine.BJYRtcEventObserver;
import com.baijiayun.bjyutils.TimeUtilsKt;
import com.baijiayun.bjyutils.reactivex.RxUtils;
import com.baijiayun.groupclassui.R;
import com.baijiayun.groupclassui.global.EventKey;
import com.baijiayun.groupclassui.global.IRouter;
import com.baijiayun.livebase.context.LPConstants;
import com.baijiayun.livecore.wrapper.LPRecorder;

import java.util.concurrent.TimeUnit;

import io.reactivex.Observable;
import io.reactivex.android.schedulers.AndroidSchedulers;
import io.reactivex.disposables.CompositeDisposable;
import io.reactivex.disposables.Disposable;

/**
 * Created by Shubo on 2018/9/20.
 */
public class StatusBarPresenter implements StatusBarContract.Presenter {

    private IRouter iRouter;
    private StatusBarContract.View view;
    private Context context;
    private Disposable disposableOfUpLinkLossRate, disposableOfDownLinkLossRate, disposableOfTimer;
    private CompositeDisposable compositeDisposable = new CompositeDisposable();

    private long startTimeTs = 0;
    private long realStartTime = 0;
    private boolean classIsStart = false;
    private boolean isStudyRoom = false;
    private String courseDuration = TimeUtilsKt.format24HoursTime(0);

    StatusBarPresenter(StatusBarContract.View view, Context context) {
        this.view = view;
        this.context = context;
    }

    @Override
    public void subscribe() {
        view.showClassName(iRouter.getLiveRoom().getRoomInfo().title);
        startTimeTs = iRouter.getLiveRoom().getRoomInfo().startTimets / 1000;
        realStartTime = startTimeTs;
        isStudyRoom = iRouter.getLiveRoom().getStudyRoomVM().isStudyRoom();
        if (iRouter.getLiveRoom().isClassStarted()) {
            startRateDisposable();
        }
        compositeDisposable.add(iRouter.getLiveRoom().getObservableOfClassStart()
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(stats -> startRateDisposable()));

        compositeDisposable.add(iRouter.getLiveRoom().getObservableOfClassEnd()
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(stats -> {
                    stopRateDisposable();
                    classIsStart = false;
                    view.showResetStatusBarText();
                }));

        compositeDisposable.add(iRouter.getLiveRoom().getObservableOfRealStartTime()
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(realTime -> realStartTime = realTime / 1000));

        compositeDisposable.add(Observable.interval(0, 1, TimeUnit.SECONDS)
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(aLong -> {
                    String classTimeStr;
                    long nowTime = System.currentTimeMillis() / 1000;
                    if (realStartTime != startTimeTs && !classIsStart) {
                        // 下课了
                        classTimeStr = context.getString(R.string.bjysc_status_bar_class_has_end)
                                + " " + courseDuration;
                    } else if (realStartTime != startTimeTs) {
                        // 正在上课
                        courseDuration = TimeUtilsKt.format24HoursTime(nowTime - realStartTime);
                        String classStartStr = isStudyRoom ? context.getString(R.string.bjysc_status_bar_class_has_use)
                                : context.getString(R.string.bjysc_status_bar_class_has_start);
                        classTimeStr = classStartStr + " " + courseDuration;
                    } else if (!classIsStart) {
                        // 未上课
                        classTimeStr = startTimeTs < nowTime
                                ? context.getString(R.string.bjysc_status_bar_overtime)
                                + " " + TimeUtilsKt.format24HoursTime(nowTime - startTimeTs)
                                : context.getString(R.string.bjysc_status_bar_distance_from_class)
                                + " " + TimeUtilsKt.format24HoursTime(startTimeTs - nowTime);
                    } else {
                        if (realStartTime != 0 && nowTime > realStartTime) {
                            courseDuration = TimeUtilsKt.format24HoursTime(nowTime - realStartTime);
                            String classStartStr = isStudyRoom ? context.getString(R.string.bjysc_status_bar_class_has_use)
                                    : context.getString(R.string.bjysc_status_bar_class_has_start);
                            classTimeStr = classStartStr + " " + courseDuration;
                        } else {
                            // 上课了，但是 realStartTime = 0，还没拿到值
                            String classStartStr = isStudyRoom ? context.getString(R.string.bjysc_status_bar_class_has_use)
                                    : context.getString(R.string.bjysc_status_bar_class_has_start);
                            classTimeStr = classStartStr + " " + TimeUtilsKt.format24HoursTime(0);
                        }
                    }
                    view.showClassTime(classTimeStr);
                }));
    }

    private boolean isUpLinkChanged, isDownLinkChanged;
    private LPConstants.MediaNetworkQuality worstUpNetworkQuality;
    private LPConstants.MediaNetworkQuality worstDownNetworkQuality;
    private void startRateDisposable() {
        classIsStart = true;
        stopRateDisposable();
        disposableOfUpLinkLossRate = iRouter.getLiveRoom().getRecorder().getObservableOfUpPacketLossRate()
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(localStreamStats -> {
                    isUpLinkChanged = true;
                    LPRecorder recorder = iRouter.getLiveRoom().getRecorder();
                    if (recorder != null) {
                        boolean isVideoOn = recorder.isVideoAttached();
                        boolean isAudioOn = recorder.isAudioAttached();
                        double lossRate = isVideoOn ? localStreamStats.videoPacketsLostRateSent : localStreamStats.audioPacketsLostRateSent;
                        LPConstants.MediaNetworkQuality upLinkNetStatus = getNetworkQualityByLossRate(lossRate);
                        view.showUpLinkLossRate(lossRate, upLinkNetStatus);
                        double videoUpBitrate = localStreamStats.videoBitrateSent;
                        double audioUpBitrate = localStreamStats.audioBitrateSent;
                        LPConstants.MediaNetworkQuality videoUpStatus = getNetworkQualityByVideoBitRate(videoUpBitrate);
                        LPConstants.MediaNetworkQuality audioUpStatus = getNetworkQualityByAudioBitRate(audioUpBitrate);
                        view.showUpVideoBitrate(videoUpBitrate, videoUpStatus);
                        view.showUpAudioBitrate(audioUpBitrate, audioUpStatus);
                        worstUpNetworkQuality = getWorseNetworkQuality(upLinkNetStatus,getWorseNetworkQuality(videoUpStatus,audioUpStatus));
                        view.updateWorstNetworkQuality(worstUpNetworkQuality,worstDownNetworkQuality);
                    }
                });

        disposableOfDownLinkLossRate = iRouter.getLiveRoom().getPlayer().getObservableOfDownLinkLossRate()
                .buffer(1, TimeUnit.SECONDS)
                .filter(maps -> maps.size() > 0)
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(stats -> {
                    isDownLinkChanged = true;
                    double sum = 0, average;
                    double videoSum = 0, videoAvg;
                    double audioSum = 0, audioAvg;
                    for (BJYRtcEventObserver.RemoteStreamStats streamStats : stats) {
                        sum += streamStats.receivedVideoLostRate;
                        videoSum += streamStats.receivedVideoBitrate;
                        audioSum += streamStats.receivedAudioBitrate;
                    }
                    int size = stats.size();
                    average = sum / size;
                    videoAvg = videoSum / size;
                    audioAvg = audioSum / size;
                    LPConstants.MediaNetworkQuality videoDownStatus = getNetworkQualityByVideoBitRate(videoAvg);
                    LPConstants.MediaNetworkQuality audioDownStatus = getNetworkQualityByAudioBitRate(audioAvg);
                    LPConstants.MediaNetworkQuality downLinkNetStatus = getNetworkQualityByLossRate(average);
                    view.showDownLinkLossRate(average, downLinkNetStatus);
                    view.showDownVideoBitrate(videoAvg, videoDownStatus);
                    view.showDownAudioBitrate(audioAvg, audioDownStatus);
                    worstDownNetworkQuality = getWorseNetworkQuality(downLinkNetStatus,getWorseNetworkQuality(videoDownStatus,audioDownStatus));
                    view.updateWorstNetworkQuality(worstUpNetworkQuality,worstDownNetworkQuality);
                });
        disposableOfTimer = Observable.interval(5, TimeUnit.SECONDS)
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(aLong -> {
                    if (!isUpLinkChanged) {
                        view.showUpLinkLossRate(-1.0, LPConstants.MediaNetworkQuality.EXCELLENT);
                        view.showUpVideoBitrate(-1.0, LPConstants.MediaNetworkQuality.EXCELLENT);
                        view.showUpAudioBitrate(-1.0, LPConstants.MediaNetworkQuality.EXCELLENT);
                        worstUpNetworkQuality = null;
                        view.updateWorstNetworkQuality(worstUpNetworkQuality,worstDownNetworkQuality);
                    }
                    if (!isDownLinkChanged) {
                        view.showDownLinkLossRate(-1.0, LPConstants.MediaNetworkQuality.EXCELLENT);
                        view.showDownVideoBitrate(-1.0, LPConstants.MediaNetworkQuality.EXCELLENT);
                        view.showDownAudioBitrate(-1.0, LPConstants.MediaNetworkQuality.EXCELLENT);
                        worstDownNetworkQuality = null;
                        view.updateWorstNetworkQuality(worstUpNetworkQuality,worstDownNetworkQuality);
                    }
                    isUpLinkChanged = false;
                    isDownLinkChanged = false;
                });
    }

    private void stopRateDisposable() {
        RxUtils.dispose(disposableOfUpLinkLossRate);
        RxUtils.dispose(disposableOfDownLinkLossRate);
        RxUtils.dispose(disposableOfTimer);
    }

    @Override
    public void help() {
        iRouter.getSubjectByKey(EventKey.HelpWindow).onNext(true);
    }

    @Override
    public void setting() {
        iRouter.getSubjectByKey(EventKey.SettingWindow).onNext(true);
    }

    @Override
    public void exit() {
        iRouter.getSubjectByKey(EventKey.CloseDialog).onNext(true);
    }

    @Override
    public void report() {
        iRouter.getSubjectByKey(EventKey.ReportDialog).onNext(true);
    }

    @Override
    public void setRouter(IRouter router) {
        iRouter = router;
    }

    @Override
    public void unSubscribe() {
        RxUtils.dispose(disposableOfUpLinkLossRate);
        RxUtils.dispose(disposableOfDownLinkLossRate);
        RxUtils.dispose(disposableOfTimer);
        compositeDisposable.clear();
    }

    @Override
    public void destroy() {
        unSubscribe();
        compositeDisposable.dispose();
        view = null;
    }

    private LPConstants.MediaNetworkQuality getNetworkQualityByLossRate(double lossRate) {
        LPConstants.MediaNetworkQuality quality = LPConstants.MediaNetworkQuality.EXCELLENT;
        if (lossRate > 0 && lossRate < 1) {
            quality = LPConstants.MediaNetworkQuality.EXCELLENT;
        } else if (lossRate >= 1 && lossRate < 5) {
            quality = LPConstants.MediaNetworkQuality.GOOD;
        } else if (lossRate >= 5 && lossRate < 10) {
            quality = LPConstants.MediaNetworkQuality.BAD;
        } else if (lossRate >= 10) {
            quality = LPConstants.MediaNetworkQuality.TERRIBLE;
        }
        return quality;
    }

    private LPConstants.MediaNetworkQuality getNetworkQualityByVideoBitRate(double videoBitRate) {
        LPConstants.MediaNetworkQuality quality = LPConstants.MediaNetworkQuality.EXCELLENT;
        if (videoBitRate > 0 && videoBitRate < 50) {
            quality = LPConstants.MediaNetworkQuality.TERRIBLE;
        } else if (videoBitRate >= 50 && videoBitRate < 100) {
            quality = LPConstants.MediaNetworkQuality.BAD;
        } else if (videoBitRate >= 100 && videoBitRate < 300) {
            quality = LPConstants.MediaNetworkQuality.GOOD;
        } else if (videoBitRate >= 300) {
            quality = LPConstants.MediaNetworkQuality.EXCELLENT;
        }
        return quality;
    }

    private LPConstants.MediaNetworkQuality getNetworkQualityByAudioBitRate(double audioBitRate) {
        LPConstants.MediaNetworkQuality quality = LPConstants.MediaNetworkQuality.EXCELLENT;
        if (audioBitRate > 0 && audioBitRate < 10) {
            quality = LPConstants.MediaNetworkQuality.TERRIBLE;
        } else if (audioBitRate >= 10 && audioBitRate < 15) {
            quality = LPConstants.MediaNetworkQuality.BAD;
        } else if (audioBitRate >= 15 && audioBitRate < 20) {
            quality = LPConstants.MediaNetworkQuality.GOOD;
        } else if (audioBitRate >= 20) {
            quality = LPConstants.MediaNetworkQuality.EXCELLENT;
        }
        return quality;
    }

    private LPConstants.MediaNetworkQuality getWorseNetworkQuality(
            LPConstants.MediaNetworkQuality quality1,
            LPConstants.MediaNetworkQuality quality2
    ) {
        if(quality1.getQuality()>quality2.getQuality()){
            return quality1;
        }else{
            return quality2;
        }
    }
}
