package com.baijiayun.groupclassui.window.study;

import android.animation.ObjectAnimator;
import android.content.Context;
import android.text.TextUtils;
import android.view.View;
import android.view.animation.Animation;
import android.view.animation.LinearInterpolator;
import android.widget.LinearLayout;

import com.baijiayun.bjyutils.reactivex.RxUtils;
import com.baijiayun.bjyutils.screen.DisplayUtils;
import com.baijiayun.groupclassui.R;
import com.baijiayun.groupclassui.global.EventKey;
import com.baijiayun.groupclassui.window.InteractiveBaseWindow;
import com.baijiayun.groupclassui.window.multiupload.IMultiUploadListener;
import com.baijiayun.groupclassui.window.multiupload.MultiUploadWindow;

import io.reactivex.android.schedulers.AndroidSchedulers;
import io.reactivex.disposables.Disposable;

/**
 * Created by pengwenjian on 2021/3/5
 * 提问页面
 * Describe:
 */
public class QuickQuestionWindow extends InteractiveBaseWindow {
    private Disposable disposableOfSubmitQuestion, disposableOfQuestion,
            disposableOfTutorAnswer, disposableOfRecallQuestion;
    private ObjectAnimator objectAnimator;
    private final IMultiUploadListener multiUploadWindow;
    private boolean hasInit = false;
    private QuickQuestionState questionState = QuickQuestionState.Edit;
    private boolean isShowing = true;

    public QuickQuestionWindow(Context context) {
        super(context);
        multiUploadWindow = new MultiUploadWindow(context);
        DisplayUtils.setRoundCorner(view, context.getResources().getDimensionPixelSize(R.dimen.bjy_base_common_bg_radius));
        LinearLayout container = $.id(R.id.ll_container).view();
        container.addView(multiUploadWindow.getView(), LinearLayout.LayoutParams.MATCH_PARENT, LinearLayout.LayoutParams.MATCH_PARENT);
        refreshQuestionState();
    }

    public QuickQuestionState getQuestionState() {
        return questionState;
    }

    public boolean isShowing() {
        return isShowing;
    }

    public void setShowing(boolean showing) {
        isShowing = showing;
    }

    public void refreshQuestionState() {
        disposableOfQuestion = iRouter.getLiveRoom().getStudyRoomVM()
                .getObservableOfQuestionNotAnswer()
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(lpStudyRoomQuestionModel -> {
                    $.id(R.id.layout_loading).gone();
                    if (!"-1".equals(lpStudyRoomQuestionModel.getId())) {
                        questionState = QuickQuestionState.Published;
                        changeState();
                        multiUploadWindow.setUploaded(true,true);
                        multiUploadWindow.setContent(lpStudyRoomQuestionModel.getQuestionContent());
                        multiUploadWindow.setRemoteImgPaths(lpStudyRoomQuestionModel.getQuestionPic());
                    } else {
                        questionState = QuickQuestionState.Edit;
                        changeState();
                        multiUploadWindow.setUploaded(false);
                        multiUploadWindow.setContent("");
                        multiUploadWindow.setRemoteImgPaths(lpStudyRoomQuestionModel.getQuestionPic());
                        multiUploadWindow.refresh();
                    }
                    if (!hasInit) {
                        hasInit = true;
                        initView();
                    }
                });
    }

    private void changeState() {
        switch (questionState) {
            case Edit:
                $.id(R.id.tv_img_desc).visible();
                $.id(R.id.tv_cancel).visible().text(getString(R.string.bjysc_cancel)).enable(true);
                $.id(R.id.tv_publish).visible().text(getString(R.string.base_answer_publish));
                $.id(R.id.tv_question_replay_tip).gone();
                $.id(R.id.tv_question_record).gone();
                $.id(R.id.tv_question_wait_reply).gone();
                break;
            case Published:
                $.id(R.id.tv_img_desc).gone();
                $.id(R.id.tv_cancel).gone();
                $.id(R.id.tv_publish).visible().text(getString(R.string.bjysc_revoke));
                $.id(R.id.tv_question_replay_tip).gone();
                $.id(R.id.tv_question_wait_reply)
                        .text(getString(R.string.bjysc_study_room_question_published_please_wait))
                        .visible();
                $.id(R.id.tv_question_record).gone();
                break;
            case Replyed:
                $.id(R.id.tv_img_desc).gone();
                $.id(R.id.tv_cancel).visible().text(getString(R.string.bjysc_revoke)).enable(false);
                $.id(R.id.tv_publish).gone();
                $.id(R.id.tv_question_wait_reply).gone();
                $.id(R.id.tv_question_replay_tip)
                        .text(getString(R.string.bjysc_study_room_question_replyed_please_check))
                        .visible();
                $.id(R.id.tv_question_record).visible();
                break;
            case Publishing:
            default:
                break;
        }
    }

    private void initView() {
        $.id(R.id.iv_close).clicked(v -> iRouter.getSubjectByKey(EventKey.DisplayStudyQuestion).onNext(DisplayStudyQuestion.Quick_Question_Close));
        $.id(R.id.tv_cancel).clicked(v -> iRouter.getSubjectByKey(EventKey.DisplayStudyQuestion).onNext(DisplayStudyQuestion.Quick_Question_Close));
        $.id(R.id.tv_publish).clicked(v -> {
            if (questionState == QuickQuestionState.Edit) {
                final String content = multiUploadWindow.getContent();
                if (TextUtils.isEmpty(content) || content.length() < 10) {
                    showToastMessage(context.getString(R.string.bjysc_enter_limit_more, 10));
                    return;
                }
                iRouter.getSubjectByKey(EventKey.DisplayStudyQuestion).onNext(DisplayStudyQuestion.Quick_Question_Close);
                RxUtils.dispose(disposableOfSubmitQuestion);
                $.id(R.id.layout_loading).visible();
                objectAnimator = ObjectAnimator.ofFloat($.id(R.id.iv_loading).view(), "rotation", 360);
                objectAnimator.setRepeatCount(Animation.INFINITE);
                objectAnimator.setInterpolator(new LinearInterpolator());
                objectAnimator.setDuration(2000);
                objectAnimator.start();
                questionState = QuickQuestionState.Publishing;
                disposableOfSubmitQuestion = iRouter.getLiveRoom().getStudyRoomVM().getObservableOfSubmitQuestion(multiUploadWindow.getRemoteImgPaths(), content, multiUploadWindow.getLocalImgPaths())
                        .observeOn(AndroidSchedulers.mainThread())
                        .subscribe(aBoolean -> {
                            String msg = aBoolean ? getString(R.string.bjysc_study_room_quick_question_publish_success)
                                    : getString(R.string.bjysc_study_room_quick_question_publish_fail);
                            iRouter.getSubjectByKey(EventKey.ReminderMessage).onNext(msg);
                            $.id(R.id.layout_loading).gone();
                            objectAnimator.cancel();
                            refreshQuestionState();
                        });
            } else if (questionState == QuickQuestionState.Published) {
                //撤回
                disposableOfRecallQuestion = iRouter.getLiveRoom().getStudyRoomVM().getObservableOfRecallQuestion()
                        .observeOn(AndroidSchedulers.mainThread())
                        .subscribe(aBoolean -> {
                            questionState = QuickQuestionState.Edit;
                            changeState();
                            multiUploadWindow.setUploaded(false);
                            multiUploadWindow.refresh();
                        });
            }
        });
        $.id(R.id.tv_question_record).clicked(v -> {
            iRouter.getSubjectByKey(EventKey.DisplayStudyQuestion).onNext(DisplayStudyQuestion.Quick_Question_Close);
            iRouter.getSubjectByKey(EventKey.DisplayStudyQuestion).onNext(DisplayStudyQuestion.Question_Record_Open);
        });
        disposableOfTutorAnswer = iRouter.getLiveRoom().getStudyRoomVM().getObservableOfTutorAnswer()
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(lpStudyRoomTutorAnswerModel -> {
                    if (lpStudyRoomTutorAnswerModel.answerType == 1) {
                        if (isShowing) {
                            if (questionState == QuickQuestionState.Published) {
                                questionState = QuickQuestionState.Replyed;
                                changeState();
                            }
                        } else {
                            refreshQuestionState();
                        }
                    }
                });
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
        RxUtils.dispose(disposableOfSubmitQuestion);
        RxUtils.dispose(disposableOfQuestion);
        RxUtils.dispose(disposableOfTutorAnswer);
        RxUtils.dispose(disposableOfRecallQuestion);
        if (objectAnimator != null) {
            objectAnimator.cancel();
        }
    }

    @Override
    protected View onCreateView(Context context) {
        return View.inflate(context, R.layout.bjysc_window_quick_question, null);
    }
}
