package com.baijiayun.groupclassui.window.toolbox.quiz;

import android.annotation.SuppressLint;
import android.annotation.TargetApi;
import android.content.Context;
import android.graphics.Bitmap;
import android.os.Build;
import android.text.TextUtils;
import android.view.View;
import android.webkit.JavascriptInterface;
import android.widget.ProgressBar;

import androidx.core.content.ContextCompat;

import com.baijiayun.bjyutils.log.LPLogger;
import com.baijiayun.bjyutils.reactivex.RxUtils;
import com.baijiayun.groupclassui.R;
import com.baijiayun.groupclassui.window.InteractiveBaseWindow;
import com.baijiayun.livebase.widgets.dialog.CommonDialog;
import com.baijiayun.livecore.LiveSDK;
import com.baijiayun.livebase.context.LPConstants;
import com.baijiayun.livebase.models.LPJsonModel;
import com.baijiayun.livebase.models.imodels.IUserModel;
import com.baijiayun.livebase.widgets.view.webview.BJWebViewImpl;
import com.baijiayun.livecore.utils.CommonUtils;
import com.tencent.smtt.export.external.interfaces.WebResourceError;
import com.tencent.smtt.export.external.interfaces.WebResourceRequest;
import com.tencent.smtt.sdk.WebChromeClient;
import com.tencent.smtt.sdk.WebView;

import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.List;

import io.reactivex.disposables.CompositeDisposable;

/**
 * 测验
 */
public class QuizWindow extends InteractiveBaseWindow implements QuizContract.View {
    private static final String ARGS_TYPE = "1";
    private static final String WINDOW_NAME = "bjlapp";
    private IUserModel currentUserInfo;
    private String quizId;
    private String roomId;
    private String roomToken;
    private List<LPJsonModel> signalList;
    private boolean isShow;
    private boolean isUrlLoaded;
    private boolean isLoadFailed = false; //url load失败了也会调到onPageFinished
    private boolean isDestroyed = false;
    private QuizContract.Presenter presenter;
    private Context context;
    private CompositeDisposable disposables;
    private CommonDialog closeDialog;
    private BJWebViewImpl mWebView;

    public QuizWindow(Context context, boolean isShow, LPJsonModel jsonModel) {
        super(context);
        this.context = context;
        this.isShow = isShow;
        this.isUrlLoaded = false;
        disposables = new CompositeDisposable();
        presenter = new QuizPresenter(this);
        setBasePresenter(presenter);
        presenter.getCurrentUser();
        roomToken = TextUtils.isEmpty(presenter.getRoomToken()) ? "" : presenter.getRoomToken();
        setAllowTouch(true);
        if (signalList == null) {
            signalList = new ArrayList<>();
        } else {
            signalList.clear();
        }
        onArrived(jsonModel);
        initView();
    }

    protected void initView() {
        $.id(R.id.rl_close).clicked(v -> showCloseDlg());
        $.id(R.id.rl_refresh).clicked(v -> {
            WebView webView = $.id(R.id.base_webview).view();
            webView.reload();
        });
        initWebClient();
        loadUrl();
    }

    private void showCloseDlg() {
        closeDialog = new CommonDialog(context, CommonDialog.ChoiceMode.Double_Blue)
                .setMainDisplayText(getString(R.string.quiz_dialog_close_tip))
                .setNegative(getString(R.string.bjysc_cancel), ((dialog, which) -> {
                    dialog.dismiss();
                }))
                .setPositive(getString(R.string.bjy_base_browser_close_btn_close), ((dialog, which) -> {
                    dialog.dismiss();
                    if (presenter != null) {
                        presenter.dismissDlg(isShow);
                    }
                }));
        closeDialog.show();
    }

    @SuppressLint({"SetJavaScriptEnabled", "JavascriptInterface"})
    private void initWebClient() {
        mWebView = $.id(R.id.base_webview).view();
        mWebView.setBackgroundColor(ContextCompat.getColor(context, R.color.bjysc_light_gray));
        mWebView.enableJavaScript(this, WINDOW_NAME);

        //chrome client
        mWebView.setWebChromeClient(new WebChromeClient() {
            @Override
            public void onProgressChanged(WebView view, int newProgress) {
                if (isDestroyed) return;
                ((ProgressBar) $.id(R.id.pb_web_view_quiz).view()).setProgress(newProgress);
                super.onProgressChanged(view, newProgress);
            }
        });

        //webView client
        mWebView.setWebViewClient(new BJWebViewImpl.BJWebViewClient() {
            @Override
            public void onPageStarted(WebView view, String url, Bitmap favicon) {
                super.onPageStarted(view, url, favicon);
                if (isDestroyed) return;
                $.id(R.id.pb_web_view_quiz).visible();
                setCloseBtnStatus(isShow);
            }

            @Override
            public void onPageFinished(WebView view, String url) {
                super.onPageFinished(view, url);
                if (isDestroyed) return;
                isUrlLoaded = true;
                $.id(R.id.pb_web_view_quiz).gone();
                if (!isLoadFailed) {
                    setCloseBtnStatus(isShow);
                }
                isLoadFailed = false;
                callJsInQueue();
            }

            @TargetApi(Build.VERSION_CODES.M)
            @Override
            public void onReceivedError(WebView view, WebResourceRequest request, WebResourceError error) {
                super.onReceivedError(view, request, error);
                // code -11 net::ERR_BAD_SSL_CLIENT_AUTH_CERT 忽略https证书问题
                if (error.getErrorCode() != -11) {
                    $.id(R.id.pb_web_view_quiz).gone();
                    setCloseBtnStatus(isShow);
                    isLoadFailed = true;
                }
            }

            @Override
            public void onReceivedError(WebView view, int errorCode, String description, String failingUrl) {
                super.onReceivedError(view, errorCode, description, failingUrl);
                if (errorCode != -11) {
                    $.id(R.id.pb_web_view_quiz).gone();
                    setCloseBtnStatus(isShow);
                    isLoadFailed = true;
                }
            }
        });
    }

    //是否显示关闭按钮
    public void setCloseBtnStatus(boolean isShow) {
        if (presenter == null) return;
        $.id(R.id.rl_close).visibility(isShow ? View.VISIBLE : View.INVISIBLE);
        $.id(R.id.rl_refresh).visibility(View.INVISIBLE);
    }

    private void loadUrl() {
        try {
            roomToken = URLEncoder.encode(roomToken, "utf-8");
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }
        String params = "?userNumber=" + currentUserInfo.getNumber() + "&userName=" + CommonUtils.encodeUTF8(currentUserInfo.getName()) + "&quizId=" + quizId
                + "&roomId=" + roomId + "&token=" + roomToken + "&argType=" + ARGS_TYPE + "&userGroup=" + currentUserInfo.getGroup() + "&skin=dark";
        String host = LPConstants.HOSTS_WEB[LiveSDK.getDeployType().getType()];
        String url = host.concat("m/quiz/student").concat(params);
        LPLogger.i(getClass().getSimpleName() + " : " + url);
        ((WebView) $.id(R.id.base_webview).view()).loadUrl(url);
    }

    @Override
    public void onArrived(LPJsonModel jsonModel) {
        String key = jsonModel.data.get("message_type").getAsString();
        if (!jsonModel.data.has("quiz_id")) {
            quizId = "";
        } else {
            quizId = jsonModel.data.get("quiz_id").getAsString();
        }
        if (!jsonModel.data.has("class_id")) {
            roomId = "";
        } else {
            roomId = jsonModel.data.get("class_id").getAsString();
        }
        signalList.add(jsonModel);
    }

    @Override
    public void onEndArrived(LPJsonModel jsonModel) {
        String key = jsonModel.data.get("message_type").getAsString();
        if (!"quiz_end".equals(key)) {
            return;
        }
        signalList.add(jsonModel);
        /*只有load完之后才去转发h5，否则加入队列，load完成后依次转发*/
        if (isUrlLoaded) {
            callJs(jsonModel.data.toString());
        }
    }

    @Override
    public void onGetCurrentUser(IUserModel userModel) {
        this.currentUserInfo = userModel;
    }

    /**
     * 逐个给js转发信令
     */
    private void callJsInQueue() {
        if (signalList != null && signalList.size() > 0) {
            for (LPJsonModel jsonModel : signalList) {
                callJs(jsonModel.data.toString());
            }
        }
    }

    @Override
    public void onDestroy() {
        mWebView.destroy();
        if (closeDialog != null) {
            closeDialog.dismiss();
        }
        isDestroyed = true;
        if (signalList != null) {
            signalList.clear();
        }
        RxUtils.dispose(disposables);
        super.onDestroy();
    }

    /******************
     * js call android
     ******************/
    @JavascriptInterface
    public void close() {
        if (presenter.checkRouterNull()) return;
        presenter.dismissDlg(isShow);
    }

    @JavascriptInterface
    public void sendMessage(String json) {
        if (!TextUtils.isEmpty(json)) {
            presenter.submitAnswer(json);
        }
    }

    /*************
     * android call js
     ************/
    private void callJs(String json) {
        ((WebView) $.id(R.id.base_webview).view()).loadUrl("javascript:bjlapp.receivedMessage(" + json + ")");
    }

    @Override
    public void setPresenter(QuizContract.Presenter presenter) {
        // do nothing
    }

    @Override
    protected View onCreateView(Context context) {
        return View.inflate(context, R.layout.bjy_group_window_quiz, null);
    }
}
