package com.baijiayun.groupclassui.window.toolbox.randomselect;

import android.content.Context;
import android.graphics.Color;
import android.view.LayoutInflater;
import android.widget.ListView;

import androidx.core.content.ContextCompat;

import com.baijiayun.bjyutils.drawable.DrawableBuilder;
import com.baijiayun.bjyutils.drawable.LayerDrawableBuilder;
import com.baijiayun.bjyutils.kt.UtilsKt;
import com.baijiayun.groupclassui.R;
import com.baijiayun.groupclassui.global.EventKey;
import com.baijiayun.groupclassui.widget.AutoScrollAdapter;
import com.baijiayun.groupclassui.window.BaseTitledWindow;
import com.baijiayun.livebase.utils.ThemeDataUtil;
import com.baijiayun.livecore.models.LPRandomSelectValueModel;

import org.jetbrains.annotations.NotNull;

import java.util.ArrayList;
import java.util.List;

public class RandomSelectWindow extends BaseTitledWindow {
    private AutoScrollAdapter autoScrollAdapter;

    private boolean isOnMission = false;
    private int targetIndex = -1;

    public RandomSelectWindow(Context context) {
        super(context);
        initUi(context);
    }

    private void initUi(Context context) {
        title(context.getResources().getString(R.string.bjysc_tool_box_random_select));
        allowTouch = true;
        ListView lvNames = $.id(R.id.bjysc_random_select_names_container).view();

        autoScrollAdapter = AutoScrollAdapter.initListView(lvNames, context);
        autoScrollAdapter.setMaxNum(3);
        autoScrollAdapter.setTextColor(ThemeDataUtil.getColorFromThemeConfigByAttrId(context, R.attr.base_theme_window_assistant_text_color)
                , ThemeDataUtil.getColorFromThemeConfigByAttrId(context, R.attr.base_theme_window_main_text_color));
        autoScrollAdapter.setOnAutoScrollListener(() -> {
            if (isDestroy()) {
                return;
            }
            isOnMission = false;
            $.id(R.id.bjysc_random_select_operate_btn).visible();
        });

        $.id(R.id.bjysc_random_select_operate_btn).clicked(v ->
                iRouter.getSubjectByKey(EventKey.OpenRandomSelectWindow).onNext(new LPRandomSelectValueModel()));

        setCanScale(false);
        showScaleIcon(false);
        showClose(false);
        showFull(false);
        showSync(false);
        hideContentMask();

        initDrawable();
    }

    /**
     * 初始化数据
     * @param randomSelectValueModel 随机选人数据集合
     */
    public void initValues(@NotNull LPRandomSelectValueModel randomSelectValueModel) {
        List<String> names = new ArrayList<>();
        targetIndex = -1;
        for (int i = 0; i < randomSelectValueModel.nameList.length; i++) {
            names.add(randomSelectValueModel.nameList[i]);
            String selectName = randomSelectValueModel.selectUser.name;
            if (selectName != null && selectName.equals(randomSelectValueModel.nameList[i])) {
                targetIndex = i;
                break;
            }
        }
        if (targetIndex == -1) {
            names.add(randomSelectValueModel.selectUser.name);
            targetIndex = names.size() - 1;
        }
        autoScrollAdapter.initValues(names);
    }

    /**
     * 开始旋转
     */
    public void startMission() {
        if (!isOnMission) {
            $.id(R.id.bjysc_random_select_operate_btn).invisible();
            isOnMission = true;
            view.postDelayed(() ->
                    autoScrollAdapter.updateSelect(targetIndex)
                    , 100
            );
        }
    }

    private void initDrawable() {
        $.id(R.id.bjysc_random_select_border_container).view().setBackground(new LayerDrawableBuilder()
                .add(new DrawableBuilder().solidColor(Color.TRANSPARENT)
                        .cornerRadius(UtilsKt.getDp(8))
                        .strokeColor(ThemeDataUtil.getColorFromThemeConfigByAttrId(context, R.attr.base_theme_window_bg_color))
                        .strokeWidth(UtilsKt.getDp(8)).build())
                .add(new DrawableBuilder().solidColor(Color.TRANSPARENT)
                        .cornerRadius(UtilsKt.getDp(8))
                        .strokeColor(ContextCompat.getColor(context, R.color.bjysc_bg_stroke))
                        .strokeWidth(1)
                        .build()
                ).build());
    }

    @Override
    public boolean onRoomStatusChange(boolean isActive) {
        boolean isWindowInit = super.onRoomStatusChange(isActive);
        if (isWindowInit) {
            if (isActive) {
                subscribe();
            } else {
                onDestroy();
            }
        }
        return true;
    }

    private void subscribe() {
        // do nothing
    }

    @Override
    protected void onCreateContentView(Context context) {
        LayoutInflater.from(context).inflate(R.layout.bjysc_window_random_select, rlContentContainer);
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
    }
}
