package com.baijiayun.groupclassui.window.toolbox.smallblackboard;

import android.content.Context;
import android.graphics.drawable.Drawable;
import android.util.AttributeSet;
import android.view.Gravity;
import android.view.MotionEvent;
import android.view.View;
import android.widget.Button;
import android.widget.LinearLayout;
import android.widget.RelativeLayout;
import android.widget.TextView;

import androidx.core.content.ContextCompat;

import com.baijiayun.bjyutils.drawable.DrawableBuilder;
import com.baijiayun.bjyutils.kt.UtilsKt;
import com.baijiayun.groupclassui.R;
import com.baijiayun.livebase.utils.ThemeDataUtil;

/**
 * Created by Shubo on 2019/3/27.
 */
class SmallBlackboardDialog extends RelativeLayout {

    private Button confirm, cancel;
    private TextView content;

    SmallBlackboardDialog(Context context) {
        this(context, null);
    }

    SmallBlackboardDialog(Context context, AttributeSet attrs) {
        this(context, attrs, 0);
    }

    SmallBlackboardDialog(Context context, AttributeSet attrs, int defStyleAttr) {
        super(context, attrs, defStyleAttr);
        init(context);
    }

    private void init(Context context) {
        final Drawable drawable = new DrawableBuilder()
                .solidColor(ThemeDataUtil.getColorFromThemeConfigByAttrId(context, R.attr.base_theme_window_bg_color))
                .cornerRadius(context.getResources().getDimensionPixelSize(R.dimen.bjy_base_common_bg_radius))
                .strokeWidth(UtilsKt.getDp(1))
                .strokeColor(ContextCompat.getColor(context,R.color.bjysc_bg_stroke_10))
                .build();
        RelativeLayout relativeLayout = new RelativeLayout(getContext());
        relativeLayout.setGravity(Gravity.CENTER);
        relativeLayout.setBackground(drawable);

        content = new TextView(getContext());
        confirm = new Button(getContext());
        cancel = new Button(getContext());

        confirm.setText(R.string.bjysc_confirm);
        cancel.setText(R.string.bjysc_cancel);

        confirm.setTextSize(12);
        cancel.setTextSize(12);
        confirm.setGravity(Gravity.CENTER);
        confirm.setPadding(0, 0, 0, 0);
        cancel.setGravity(Gravity.CENTER);
        cancel.setPadding(0,0,0,0);
        confirm.setBackground(new DrawableBuilder()
                .solidColor(ThemeDataUtil.getColorFromThemeConfigByAttrId(context, R.attr.base_theme_live_product_color))
                .cornerRadius(context.getResources().getDimensionPixelSize(R.dimen.bjy_base_common_bg_radius))
                .build());
        cancel.setBackground(new DrawableBuilder()
                .solidColor(ThemeDataUtil.getColorFromThemeConfigByAttrId(context, R.attr.base_theme_dialog_negative_bg_color))
                .cornerRadius(context.getResources().getDimensionPixelSize(R.dimen.bjy_base_common_bg_radius))
                .build());

        content.setTextColor(ThemeDataUtil.getColorFromThemeConfigByAttrId(getContext(), R.attr.base_theme_window_main_text_color));
        confirm.setTextColor(ThemeDataUtil.getColorFromThemeConfigByAttrId(getContext(), R.attr.base_theme_dialog_positive_text_color));
        cancel.setTextColor(ThemeDataUtil.getColorFromThemeConfigByAttrId(getContext(), R.attr.base_theme_dialog_negative_text_color));

        content.setGravity(Gravity.CENTER);
        content.setId(View.generateViewId());
        relativeLayout.addView(content, new LayoutParams(LayoutParams.MATCH_PARENT, LayoutParams.WRAP_CONTENT));

        LinearLayout linearLayout = new LinearLayout(getContext());
        linearLayout.setOrientation(LinearLayout.HORIZONTAL);

        LayoutParams cancelButtonLayout = new LayoutParams((int)getResources().getDimension(R.dimen.bjysc_sbb_dialog_btn_width), (int)getResources().getDimension(R.dimen.bjysc_sbb_dialog_btn_height));
        linearLayout.addView(cancel, cancelButtonLayout);
        LayoutParams confirmButtonLayout = new LayoutParams((int)getResources().getDimension(R.dimen.bjysc_sbb_dialog_btn_width), (int)getResources().getDimension(R.dimen.bjysc_sbb_dialog_btn_height));
        confirmButtonLayout.leftMargin = UtilsKt.getDp(24);
        linearLayout.addView(confirm, confirmButtonLayout);

        LayoutParams linearParam = new LayoutParams(LayoutParams.WRAP_CONTENT, LayoutParams.WRAP_CONTENT);
        linearParam.addRule(RelativeLayout.BELOW, content.getId());
        linearParam.addRule(CENTER_HORIZONTAL);
        linearParam.topMargin = UtilsKt.getDp(16);

        relativeLayout.addView(linearLayout, linearParam);

        LayoutParams layoutParams = new LayoutParams((int) getResources().getDimension(R.dimen.bjysc_sbb_dialog_width), (int) getResources().getDimension(R.dimen.bjysc_sbb_dialog_height));
        layoutParams.addRule(CENTER_IN_PARENT);
        addView(relativeLayout, layoutParams);
    }

    SmallBlackboardDialog setContent(String s) {
        content.setText(s);
        return this;
    }

    SmallBlackboardDialog setOnConfirmClickListener(OnClickListener onClickListener) {
        confirm.setOnClickListener(onClickListener);
        return this;
    }

    SmallBlackboardDialog setOnCancelClickListener(OnClickListener onClickListener) {
        cancel.setOnClickListener(onClickListener);
        return this;
    }

    @Override
    public boolean onTouchEvent(MotionEvent event) {
        return true;
    }
}
