package com.baijiayun.groupclassui.window.toolbox.smallblackboard;

import android.graphics.Color;
import android.graphics.drawable.ColorDrawable;
import android.graphics.drawable.Drawable;
import android.view.Gravity;
import android.view.View;
import android.view.ViewGroup;
import android.view.ViewParent;
import android.widget.LinearLayout;
import android.widget.PopupWindow;
import android.widget.RelativeLayout;
import android.widget.TextView;

import androidx.core.content.ContextCompat;

import com.baijiayun.bjyutils.drawable.DrawableBuilder;
import com.baijiayun.bjyutils.kt.UtilsKt;
import com.baijiayun.groupclassui.R;
import com.baijiayun.livebase.utils.ThemeDataUtil;

/**
 * Created by Shubo on 2019/3/27.
 */
public class SmallBlackboardDialogUtil {

    public static void showDialog(View rootView, String content, View.OnClickListener onConfirmClickListener) {
        showDialog(rootView, content, onConfirmClickListener, null);
    }

    public static View showTips(View rootView, String content){
        return showTips(rootView, content, 0);
    }

    public static View showTips(View rootView, String content, int delay){
        final Drawable drawable = new DrawableBuilder()
                .solidColor(ThemeDataUtil.getColorFromThemeConfigByAttrId(rootView.getContext(), R.attr.base_theme_window_bg_color))
                .cornerRadius(rootView.getContext().getResources().getDimensionPixelSize(R.dimen.bjy_base_common_bg_radius))
                .strokeWidth(UtilsKt.getDp(1))
                .strokeColor(ContextCompat.getColor(rootView.getContext(),R.color.bjysc_bg_stroke_10))
                .build();
        LinearLayout linearLayout = new LinearLayout(rootView.getContext());
        linearLayout.setBackground(drawable);
        linearLayout.setGravity(Gravity.CENTER);
        TextView textView = new TextView(rootView.getContext());
        int _12dp = UtilsKt.getDp(12);
        int _4dp = UtilsKt.getDp(4);
        textView.setId(R.id.lp_sbb_tip_content_tv);
        textView.setPadding(_12dp, _4dp, _12dp, _4dp);
        textView.setText(content);
        textView.setTextColor(ThemeDataUtil.getColorFromThemeConfigByAttrId(rootView.getContext()
                , R.attr.base_theme_window_main_text_color));
        linearLayout.addView(textView);
        RelativeLayout.LayoutParams layoutParams = new RelativeLayout.LayoutParams(ViewGroup.LayoutParams.WRAP_CONTENT, ViewGroup.LayoutParams.WRAP_CONTENT);
        layoutParams.addRule(RelativeLayout.ALIGN_PARENT_RIGHT);
        layoutParams.addRule(RelativeLayout.ALIGN_PARENT_TOP);
        layoutParams.topMargin = UtilsKt.getDp(42);
        layoutParams.rightMargin = _12dp;
        ((ViewGroup) rootView).addView(linearLayout, layoutParams);
        if(delay > 0){
            rootView.postDelayed(() -> ((ViewGroup) rootView).removeView(linearLayout), delay * 1000);
        }
        return linearLayout;
    }

    public static void showDialog(View rootView, String content, View.OnClickListener onConfirmClickListener,
                           View.OnClickListener onCancelClickListener) {
        if (!(rootView instanceof ViewGroup)) {
            return;
        }
        SmallBlackboardDialog dialog = new SmallBlackboardDialog(rootView.getContext())
                .setContent(content)
                .setOnConfirmClickListener(v -> {
                    removeView(rootView, v);
                    if (onConfirmClickListener != null)
                        onConfirmClickListener.onClick(v);
                })
                .setOnCancelClickListener(v -> {
                    removeView(rootView, v);
                    if (onCancelClickListener != null)
                        onCancelClickListener.onClick(v);
                });
        ((ViewGroup) rootView).addView(dialog, new ViewGroup.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.MATCH_PARENT));
    }

    private static void removeView(View rootView, View view) {
        while (view.getParent() != rootView) {
            ViewParent viewParent = view.getParent();
            if (!(viewParent instanceof View)) return;
            view = (View) view.getParent();
        }
        if (rootView instanceof ViewGroup)
            ((ViewGroup) rootView).removeView(view);
    }

    static PopupWindow showPopupWindow(View anchorView, String content){
        PopupWindow popupWindow = new PopupWindow();
        popupWindow.setHeight(ViewGroup.LayoutParams.WRAP_CONTENT);
        popupWindow.setWidth(ViewGroup.LayoutParams.WRAP_CONTENT);
        popupWindow.setOutsideTouchable(true);
        popupWindow.setFocusable(true);
        popupWindow. setBackgroundDrawable(new ColorDrawable(Color.TRANSPARENT));
        TextView textView = new TextView(anchorView.getContext());
        textView.setText(content);
        textView.setTextColor(Color.parseColor("#ffededee"));
        popupWindow.setContentView(textView);
        return popupWindow;
    }
}
